
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * HEADER cfg.h
 *
 * PURPOSE:
 *	This header file is the configuration utiliy include file.  
 *
 * SCOPE:
 *	Only the cfg utilities users.exe and targets.exe require this header.
 *	
 * NOTES:
 *	Character pointers specified in any structure element are null 
 *	terminated ASCII strings unless a count is associated with it.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 * 
 ****************************************************************************/

#ifndef CFG_H

#define CFG_H

#include <stdio.h>
#include <string.h>

#define DOS

/* Basic types */
typedef char 			CHAR;
typedef unsigned char 	UCHAR;
typedef short 			SHORT;
typedef unsigned short	USHORT;
typedef long 			LONG;
typedef unsigned long	ULONG;
#ifndef BOOLEAN
typedef SHORT			BOOLEAN;
#endif
#ifndef FALSE
#define FALSE			0
#endif
#ifndef TRUE
#define TRUE			1 
#endif
#ifndef EOS
#define EOS				'\0'
#endif

typedef struct
{
	CHAR	*label;
	SHORT	tag;
} CFG_XREF;


extern SHORT Cfg_getTag(CFG_XREF *xrefTable, SHORT xrefTableSize,
					   CHAR *rec, CHAR **recData);

extern CHAR *Cfg_getData(char *data);


#define CFG_MAX_FILE_NAME_SIZE 80

#define CFG_TAG_LENGTH		4
#define CFG_STATUS_LENGTH	1
#define CFG_KEY_LENGTH		80
#define CFG_LEN_LENGTH		10
#define CFG_MAX_RECORD_SIZE	4000

#define CFG_RECORD_TERM	"\n"
#define CFG_NULL_KEY_S	""
#define CFG_NULL_TAG	0
#define CFG_BLANK_LINE	-1

typedef struct
{
	CHAR	tag[CFG_TAG_LENGTH+1];
	CHAR	status[CFG_STATUS_LENGTH+1];
	CHAR	key[CFG_KEY_LENGTH+1];
	CHAR	length[CFG_LEN_LENGTH+1];	/* total of all CFG_RECORD_FIELD */
	CHAR	*data;						/* series of CFG_RECORD_FIELD */
} CFG_RECORD;

typedef struct
{
	CHAR	tag[CFG_TAG_LENGTH+1];
	CHAR	length[CFG_LEN_LENGTH+1];	/* length of field data */
	CHAR	*data;
} CFG_RECORD_FIELD;



extern void Cfg_initRecord(CFG_RECORD *record, CHAR *tag, CHAR *key,
						   CHAR *data);

extern void Cfg_setRecordKey(CFG_RECORD *record, CHAR *key);

extern void Cfg_initField(CFG_RECORD_FIELD *field, SHORT tag, CHAR *data);

extern void Cfg_appendField(CFG_RECORD *record, CFG_RECORD_FIELD *field);

extern void Cfg_writeRecord(FILE *fp, CFG_RECORD *record);




#define CFG_REC_OK 		"0"
#define CFG_REC_DELETED	"1" 


#define CFG_USER_INFO		"A1A1"	
#define CFG_TARGET_INFO		"A2A2"
#define CFG_DATABASE_INFO	"A3A3" 

#define CFG_USER		1
#define CFG_TARGET		2
#define CFG_DATABASE	3

#define CFG_USER_NAME	50
#define CFG_PASSWORD	51
#define CFG_PRIVILEGE	52
#define CFG_PREF_LANG	53
#define CFG_CREDIT		54
#define CFG_TRGT_FILE	55

#define CFG_TRGT_ID		60
#define CFG_TRGT_NAME 	61
#define CFG_USER_AUTH	62
#define CFG_UPR_BND		63
#define CFG_LWR_BND		64
#define CFG_PRSNT_NUM	65
#define CFG_REC_SYN		66
#define CFG_SET_NAME	67
#define CFG_DSPLY_FMT	68
#define CFG_DESC		69
#define	CFG_OPTIONS		70
#define CFG_CNTCT_INFO	71
#define CFG_RSLT_SET	72
#define	CFG_MAX_RSLT_SET 73
#define CFG_HRS_OP		74
#define CFG_CHRG_POLICY	75
#define CFG_ID_AUTH		77
#define CFG_CNTX_NAME	78
#define CFG_PRCSS_TITLE	79
#define CFG_ENTTY_QUAL	80
#define CFG_IP_ADDR		81
#define CFG_IP_DMN_NAME	82
#define CFG_PORT_NUM	83

#define CFG_DB_NAME		90
#define CFG_ATTR_SET_ID	91

#endif
