/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log: user.c,v $
 * Revision 1.1  1993/11/19  21:25:27  irorigin
 * Initial revision
 *
 ************************************************************************/

#include "cfg.h"

static char FMT[50];

SHORT Cfg_getTag(CFG_XREF *xrefTable, SHORT xrefTableSize,
				CHAR *rec, CHAR **recData)
{
	SHORT   i;
	CHAR	*s;

	/* Skip leading white spaces */
	s = rec;
	while ((*s != EOS) && (isspace(*s)))
	{
		s++;
	}
	if (*s == EOS)
	{
		return(CFG_BLANK_LINE);
	}

	for (i = 0; i < xrefTableSize; i++)
	{
		if (strncmp(xrefTable[i].label, s, 
		    strlen(xrefTable[i].label)) == 0)
		{
			*recData = &rec[strlen(xrefTable[i].label)];
			return (xrefTable[i].tag);
		}
	}

	return (CFG_NULL_TAG);
}


CHAR *Cfg_getData(char *data)
{
	CHAR 	*s, *t;

	s = data;
	/* Search for the start of term */
	while ((*s != EOS) && (isspace(*s)))
	{
		s++;
	}

	if (*s == EOS)
	{
		return (NULL);
	}

	/* Start of term. Find end of line */
	t = s;
	while (*t != EOS)
	{
		t++;
	}
	t--;

#ifdef DOS
	if (*t == '\r')
	{
		*t = EOS;
		if (t > s)
		{
			t--;
		}
		else
		{
			return (NULL);
		}
	}
#endif

	/* Strip off excess spaces */
	while ((isspace(*t)) && (t > s))
	{
		*t = EOS;
		t--;
	}

	return(s);
}


void Cfg_initRecord(CFG_RECORD *record, CHAR *tag, CHAR *key, CHAR *data)
{
	sprintf(FMT, "%%-%0d.%0ds", CFG_TAG_LENGTH, CFG_TAG_LENGTH);
	sprintf(record->tag, FMT, tag);
	sprintf(FMT, "%%-%0d.%0ds", CFG_STATUS_LENGTH, CFG_STATUS_LENGTH);
	sprintf(record->status, FMT, CFG_REC_OK);
	sprintf(FMT, "%%-%0d.%0ds", CFG_KEY_LENGTH, CFG_KEY_LENGTH);
	sprintf(record->key, FMT, key);
	sprintf(FMT, "%%-%0dd", CFG_LEN_LENGTH);
	sprintf(record->length, FMT, 0);
	record->data = data;
	record->data[0] = EOS;
}


void Cfg_setRecordKey(CFG_RECORD *record, CHAR *key)
{
	sprintf(FMT, "%%-%0d.%0ds", CFG_KEY_LENGTH, CFG_KEY_LENGTH);
	sprintf(record->key, FMT, key);
}


void Cfg_initField(CFG_RECORD_FIELD *field, SHORT tag, CHAR *data)
{
	sprintf(FMT, "%%%0dd", CFG_TAG_LENGTH);
	sprintf(field->tag, FMT, tag);
	sprintf(FMT, "%%%0dd", CFG_LEN_LENGTH);
	sprintf(field->length, FMT, strlen(data));
	field->data = data;
}


void Cfg_appendField(CFG_RECORD *record, CFG_RECORD_FIELD *field)
{
	strcat(record->data, field->tag);
	strcat(record->data, field->length);
	strcat(record->data, field->data);

	sprintf(FMT, "%%-%0dd", CFG_LEN_LENGTH);
	sprintf(record->length, FMT, strlen(record->data));
}


void Cfg_writeRecord(FILE *fp, CFG_RECORD *record)
{
	if (strlen(record->data))
	{
		fputs(record->tag, fp);
		fputs(record->status, fp);
		fputs(record->key, fp);
		fputs(record->length, fp);
		fputs(record->data, fp);
		fputs(CFG_RECORD_TERM, fp);
	}
}
