/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log: target.c,v $
 * Revision 1.1  1993/11/19  21:25:21  irorigin
 * Initial revision
 *
 ************************************************************************/

#include "cfg.h"

/*
 * This driver is used to read in target and database information and
 * create records in the format required by the IR ORIGIN Table Management CSU.
 * These records are output to the file indicated.
 *
 * The input file is prompted for by the targets routine.
 * The format of the input data consists of:
 *      configuration file name for a single user.
 *      list of targets, where each target may be followed by a list of
 *      databases
 *
 *      - each target and database is preceded by TARGET or DATABASE
 *      - each target element is preceded by one of the following character
 *      strings
 *
 *      TARGETNAME:
 *      USERAUTHENTICATION:
 *      DEFSMALLSETUPPERBOUND:
 *      DEFLARGESETLOWERBOUND:
 *      DEFPRESENTNUMBER:
 *      DEFPREFEREDRECSYNTAX:
 *      DEFELEMENTSETNAME:
 *      DISPLAYFORMATS:
 *      DESCRIPTION:
 *      OPTIONS:
 *      CONTACTINFO:
 *      DEFRESULTSETNAMINGSUPPORTED:
 *      MAXNUMRESULTSETS:
 *      HOURSOFOPERATION:
 *      CHARGINGPOLICY:
 *      IDAUTHENTICATION:
 *      APPLCNTXNAME:
 *      APPLPROCESSTITLE:
 *      APPLENTITYQUAL:
 *      IPADDRESS:
 *      IPDOMAINNAME:
 *      PORTNUM:
 *
 *      - each database element is preceded by one of the following 
 *      character strings
 *      
 *      DATABASENAME:
 *      DBDESCRIPTION:
 *      DBCHARGINGPOLICY:
 *      ATTRSETID:
 *      RECORDSYNTAX:
 *      ELEMENTSETNAME:
 *      
 */


#define TARGET_TABLE_SIZE        25              

static CFG_XREF targetTable[TARGET_TABLE_SIZE] = {  
	{"TARGET:", CFG_TARGET},
	{"TARGETNAME:", CFG_TRGT_NAME},
	{"USERAUTHENTICATION:", CFG_USER_AUTH},
	{"DEFSMALLSETUPPERBOUND:", CFG_UPR_BND},
	{"DEFLARGESETLOWERBOUND:", CFG_LWR_BND},
	{"DEFPRESENTNUMBER:", CFG_PRSNT_NUM},
	{"DEFPREFEREDRECSYNTAX:", CFG_REC_SYN},
	{"DEFELEMENTSETNAME:", CFG_SET_NAME},
	{"DISPLAYFORMATS:", CFG_DSPLY_FMT},
	{"DESCRIPTION:", CFG_DESC},
	{"OPTIONS:", CFG_OPTIONS},
	{"CONTACTINFO:", CFG_CNTCT_INFO},
	{"DEFRESULTSETNAMINGSUPPORTED:", CFG_RSLT_SET},
	{"MAXNUMRESULTSETS:", CFG_MAX_RSLT_SET},
	{"HOURSOFOPERATION:", CFG_HRS_OP},
	{"CHARGINGPOLICY:", CFG_CHRG_POLICY},
	{"IDAUTHENTICATION:", CFG_ID_AUTH},
	{"APPLCNTXNAME:", CFG_CNTX_NAME},
	{"APPLPROCESSTITLE:", CFG_PRCSS_TITLE},
	{"APPLENTITYQUAL:", CFG_ENTTY_QUAL},
	{"IPADDRESS:", CFG_IP_ADDR},
	{"IPDOMAINNAME:", CFG_IP_DMN_NAME},
	{"PORTNUM:", CFG_PORT_NUM},
	{"DATABASE:", CFG_DATABASE}};

#define DB_TABLE_SIZE            8

static CFG_XREF dbTable[DB_TABLE_SIZE] = {  
	{"DATABASE:", CFG_DATABASE},
	{"DATABASENAME:", CFG_DB_NAME},
	{"DBDESCRIPTION:", CFG_DESC},
	{"DBCHARGINGPOLICY:", CFG_CHRG_POLICY},
	{"ATTRSETID:", CFG_ATTR_SET_ID},
	{"RECORDSYNTAX:", CFG_REC_SYN},
	{"ELEMENTSETNAME:", CFG_SET_NAME},
	{"TARGET:", CFG_TARGET}};
	

static CFG_RECORD record;
static CFG_RECORD_FIELD field;

static CHAR rec[CFG_MAX_RECORD_SIZE];
static CHAR buf[CFG_MAX_RECORD_SIZE];

static CHAR fileName[CFG_MAX_FILE_NAME_SIZE];

void main()
{
	FILE    *i_fp, *o_fp;
	CHAR    *data;
	CHAR    *dataStart;
	SHORT	tag;
	SHORT	target_id=0;
	CHAR	tmp[50];
	BOOLEAN	target, database;

	printf("Enter target information file name:\n");
	gets(fileName);
	i_fp = fopen(fileName, "r");
	if (i_fp == NULL)
	{
		printf("Error opening input file %s\n", fileName);
		exit(2);
	}

	/* get output file name */
	fgets(buf, CFG_MAX_RECORD_SIZE, i_fp);
	data = Cfg_getData(buf);
	printf("Output file is %s\n", data);

	o_fp = fopen(data, "a+");
	if (o_fp == NULL)
	{
		printf("Error opening output file %s\n", fileName);
		fclose(i_fp);
		exit(2);
	}

	target_id++;
	sprintf(tmp, "%0d", target_id);
	Cfg_initRecord(&record, CFG_TARGET_INFO, tmp, rec);
	Cfg_initField(&field, CFG_TRGT_ID, tmp);
	Cfg_appendField(&record, &field);
	target = FALSE;
	database = FALSE;
	fgets(buf, CFG_MAX_RECORD_SIZE, i_fp); 
	while (!feof(i_fp))
	{
		if (database)
		{
			tag = Cfg_getTag(dbTable, DB_TABLE_SIZE, buf, &dataStart); 
		}
		else
		{
			tag = Cfg_getTag(targetTable, TARGET_TABLE_SIZE, buf, &dataStart); 
		}
		data = Cfg_getData(dataStart);
		switch (tag)
		{
			case CFG_BLANK_LINE:
				break;

			case CFG_NULL_TAG:
				printf("invalid keyword in line:\n");
				printf("\t%s\n", buf);
				fclose(o_fp);
				fclose(i_fp);
				exit(-1);
				break;

			case CFG_TARGET:
				if (target || database)
				{
					Cfg_writeRecord(o_fp, &record);
					target_id++;
					sprintf(tmp, "%0d", target_id);
					Cfg_initRecord(&record, CFG_TARGET_INFO, tmp, rec);
					Cfg_initField(&field, CFG_TRGT_ID, tmp);
					Cfg_appendField(&record, &field);
				}
				target = TRUE;
				database = FALSE;
				break;

			case CFG_TRGT_NAME:
				if (target && data)
				{
					Cfg_initField(&field, tag, data);
					Cfg_appendField(&record, &field);
					printf("Target: %s\n", data);
				}
				break;

			case CFG_DATABASE:
				if (target || database)
				{
					Cfg_writeRecord(o_fp, &record);
					Cfg_initRecord(&record, CFG_DATABASE_INFO, 
								   CFG_NULL_KEY_S, rec);
					sprintf(tmp, "%0d", target_id);
					Cfg_initField(&field, CFG_TRGT_ID, tmp);
					Cfg_appendField(&record, &field);
				}
				target = FALSE;
				database = TRUE;
				break;

			case CFG_DB_NAME:
				if (database && data)
				{
					sprintf(tmp, "%0d%s", target_id, data);
					Cfg_setRecordKey(&record, tmp);
					Cfg_initField(&field, tag, data);
					Cfg_appendField(&record, &field);
					printf("Database: %s\n", data);
				}
				break;

			default:
				if ((target || database) && data)
				{
					Cfg_initField(&field, tag, data);
					Cfg_appendField(&record, &field);
				}
				break;
		}
		fgets(buf, CFG_MAX_RECORD_SIZE, i_fp);
	}

	if (target || database)
	{
		Cfg_writeRecord(o_fp, &record);
	}

	fclose(o_fp);
	fclose(i_fp);
	exit(1);

}
