
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC Abort Association
 *
 * DESCRIPTION:
 *	This interface function is used to abruptly terminate
 *	an association with a remote target.
 *
 * CONTENTS:
 *	IRC_AbortAssoc
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <ir_cs.h>
#include <resc.h>

/****************************************************************************
 * FUNCTION: IRC_Abort
 *
 * PURPOSE:
 *
 * INPUT:
 *
 *	assocId			unique identifier for the association.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *	This is a stub function.
 *
 ***************************************************************************/

IR_STATUS	IRC_AbortAssoc( IR_ASSOC_ID assocId )
{
	CS_SOCK_ITEM 		*sockItem;

	if (!CsCf_IsIRCInitialized())
	{
		CsCf_ReportError(IR_FILE, CS_ERR_IRC_NOT_INIT, NO_STRING,
		    NO_DATA_VALUE);
		return(IR_FAIL);
	}
	/* Get the association's state information. */
	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		/* Fake it and return success */
		return(IR_SUCCESS);
	}
	else
	{
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(assocId, (CS_SOCK_EVENT)CS_SOCK_EV_USER_ABORT);
		return (IR_SUCCESS);
	}
}
