
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC_Connect 
 *
 * DESCRIPTION:
 *	This interface unit is used to initiate a connection on a 
 *	TCP/IP network using either the host name or the address.
 *
 * CONTENTS:
 *	CsCf_Connect
 *
 * NOTES:
 *	This is a stub file.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#ifdef WINSOCKET
#include <stdlib.h>
#endif
#include <memory.h>
#include <ir_pli.h>
#include <ir_net.h>
#include <ir_cs.h>
#include <resc.h>
#ifdef BSDSOCKET
#include <fcntl.h>
#endif

/****************************************************************************
 * FUNCTION: CsCf_Connect
 *
 * PURPOSE:
 *	This function will use the input event information to determine
 *	the association which needs to be connected.
 *
 * INPUT:
 *	IR_COMM_EVENT		- communication event information.
 *
 *
 * OUTPUT:
 *	IR_STATUS	- error if invalid assocId.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void CsCf_Connect(IR_COMM_EVENT *event)
/*
 * This function will return a fail status if it could not
 * perform the connect.
 */
{
	CS_SOCK_ITEM 		*sockItem;
	HOSTENT 		*hostp;
	SOCKADDR_IN		server;
	SOCKET			socketFd;
	ULONG			nonBlocking=0;
	int			errCode;

	/* Get the association's state information. */
	sockItem = CsCf_AssocGet(event->assocId);
	if (!sockItem)
	{
		return;
	}

#ifdef WINSOCKET
	/* Check for a failed async resolve */
	if (WSAGETASYNCERROR(event->lParam))
	{
		errCode = WSAGETASYNCERROR(event->lParam);
		CS_LOG_PRINTV(IR_FILE, 
		    "Error resolving name/address. Error = %d\n",
		    errCode);
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(event->assocId, CS_SOCK_EV_FAIL_RESOLVE);
		return;
	}
#endif

	hostp = sockItem->hostp;
	sockItem->state = CS_SOCK_ST_CONNECT;

	/*
 	 * create the socket.
 	 */
	if ((socketFd = socket( hostp->h_addrtype, SOCK_STREAM, 0)) < 0)
	{
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(event->assocId, CS_SOCK_EV_ERROR_ABORT);
		 return;
	}
	
	sockItem->socketFd = socketFd;

	/* Initialize the connect structure */
	(void)memset((char *)&server, (char)NULL, sizeof(server));

	(void)memcpy((char *)&server.sin_addr,
	    (char *)hostp->h_addr, 
	    hostp->h_length);
	server.sin_family = hostp->h_addrtype;
	server.sin_port = htons((USHORT)sockItem->port);

	/*
 	 * Connect to the socket. 
	 */
#ifdef BSDSOCKET
	if (connect(socketFd, &server, sizeof(server)) < 0)
	{
		errCode = CsCf_CommErrorNo();
		if (errCode  == ECONNREFUSED)
		{
			CsCf_ReportError(IR_FILE, 
			    CS_ERR_CONN_WOULD_BLOCK,
		    	    "Connect refused", errCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(event->assocId, CS_SOCK_EV_CONN_REFUSED);
			return;
		}
		else
		{
			CsCf_ReportError(IR_FILE, 
			    CS_ERR_CONNECT_FAIL,
		    	    "Connect Failed", errCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(event->assocId, CS_SOCK_EV_ERROR_ABORT);
		}
		return;
	}
	else
	{
		/* Update the assoc list item */
		CS_LOG_PRINTV(IR_FILE, 
		    "Connect success on port num = %d\n", 
		    sockItem->port);
		/* Post a connected event */
		CsCf_Notify(event->assocId, CS_SOCK_EV_CONNECTED);
	}
#endif
#ifdef WINPATHWAY
	if (connect(socketFd, (struct sockadddr *)&server, sizeof(server)) < 0)
	{
		errCode = get_errno();
		if (errCode  == ECONNREFUSED)
		{
			CsCf_ReportError(IR_FILE, CS_ERR_CONN_WOULD_BLOCK,
			     "Connect refused", errCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(event->assocId, CS_SOCK_EV_CONN_REFUSED);
			return;
		}
		else
		{
			CsCf_ReportError(IR_FILE, 
			    CS_ERR_CONNECT_FAIL,
			    "Connect Failed", errCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(event->assocId, CS_SOCK_EV_ERROR_ABORT);
		}
	}
	else
	{
		/* Update the assoc list item */
		CS_LOG_PRINTV(IR_FILE, 
		    "Connect success on port num = %d\n", 
		    sockItem->port);
		/* Post a connected event */
		CsCf_Notify(event->assocId, CS_SOCK_EV_CONNECTED);
	}
#endif
#ifdef WINSOCKET

	/* 
	 * post message when connect is complete. This API call
	 * automatically sets the socket into non-blocking mode.
 	 */
	if (WSAAsyncSelect(socketFd, (HWND)sockItem->assocId,
    	    (unsigned int)CS_SOCK_EV_CONNECTED,
	    (long)FD_CONNECT) != 0)
	{
		CsCf_ReportError(IR_FILE, CS_ERR_SELECT,
	    	    "error in connect select. ", CsCf_CommErrorNo());
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_ERROR_ABORT);
		return;
	}

	/*
 	 * Connect to the socket.
	 */
	if (connect((SOCKET)socketFd, (SOCKADDR *)&server, sizeof(server)) != 0)
	{
		errCode = CsCf_CommErrorNo();
		if (errCode != WSAEWOULDBLOCK) 
		{
			CsCf_ReportError(IR_FILE, 
			    CS_ERR_CONNECT_FAIL,
		    	    "Connect failed", errCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(event->assocId, CS_SOCK_EV_CONN_REFUSED);
		}
		else
		{
			/* 
			 * Would block is ok. 
			 * It is in non-blocking mode.
			 */
			sockItem->state = CS_SOCK_ST_CONNECT;
		}
	}
#endif
	return;
}
