
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC Support Utilities 
 *
 * DESCRIPTION:
 *	This unit contains the functions used to verify external
 *	ASN data structures, display apdus to log file, display sbuf
 *	contents to log file.
 *
 * CONTENTS:
 *	CsCf_IRExtReset
 *	CsCf_IRExtIsOk
 *	CsCf_BadExternal
 *	CsCf_DisplayAPDU
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <apdu.h>
#include <ir_cs.h>
#include <ir_oid.h>
#include <debug.h>
#include <ir_net.h>


static BOOLEAN	CSCF_IRExtOk=TRUE;

/* 
 * This function checks for known externals. Only object identifiers
 * are accepted. Any external without an object identifier will be
 * rejected and possibly causing a presentation layer protocol error.
 */
void CsCf_BadExternal()
{

	 /* Unknown extern found. Bad external. */
	CSCF_IRExtOk = FALSE;
}

void CsCf_IRExtReset()
{

	CSCF_IRExtOk = TRUE;
}

BOOLEAN	CsCf_IRExtIsOk()
{
	
	return(CSCF_IRExtOk);
}

void CsCf_DisplayAPDU(APDU *apdu)
{
	FILE *fp;
	extern BOOLEAN RescUT_LogAPDUs;


	if (RescUT_LogAPDUs == TRUE) 
	{
		fp = fopen(RESC_UT_LOG_TMP_FILE, "a");
		if (fp != NULL)
		{
			PrintAPDU(fp, apdu, 2);
			fprintf(fp, "\n");
			fclose(fp);
		}
	}
}

/****************************************************************************
 * FUNCTION:  CsCf_DumpBufData
 *
 * PURPOSE:
 *	This function is used for debug purposes. It will dump the
 *	the contents of a character buffer to the debug Log file.
 *
 * INPUT:
 *	pointer to buffer.
 *	len of buffer in bytes.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void CsCf_DumpBufData(CHAR *buf, SHORT len)
{
	FILE *fp;
	CHAR	ch;
	CHAR	dispBuf[200];
	SHORT	i;
	SHORT	index;
	CHAR	a,b;
	SHORT	line;
	SHORT	charsPerLine;

	fp = fopen(RESC_UT_LOG_TMP_FILE, "a");

	i = 0;
	index = 0;
	line = 0;
	charsPerLine = 0;

	while (i < len)
	{
			
		ch = buf[i++];
		a = (ch & 0x0F);
		if (a < 0x0A)
		{
			a = a + 0x30;
		}
		else
		{
			a = a + 'A' - 0x0a;
		}
		b = ((ch >> 4) & 0x0F);
		if (b < 0x0A)
		{
			b = b + 0x30;
		}
		else
		{
			b = b + 'A' - 0x0a;
		}

		dispBuf[index++] = b;
		dispBuf[index++] = a;
		dispBuf[index++] = ' ';
		dispBuf[index] = '\0';
		charsPerLine++;
		
		if (charsPerLine == 25)
		{
			fprintf(fp, "%s\n", dispBuf);
			index = 0;
			dispBuf[index] = '\0';
			line++;
			if ((line % 8) == 0)
			{
				fprintf(fp, "\n");
			}
			charsPerLine = 0;
		}
	}

	fprintf(fp, "%s\n", dispBuf);
	fflush(fp);
	fclose(fp);
}

/****************************************************************************
 * LOCAL FUNCTION:  CsCf_DumpSBuf
 *
 * PURPOSE:
 *	This function will is used for debug purposes. It will dump the
 *	contents of an SBuf and the SBuf pointers to the ir log file.
 *
 * INPUT:
 *	pointer to Sbuf.
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void CsCf_DumpSBuf(CHAR *srcFile, SBuf *b)
{
	CHAR	dispStr[200];

	sprintf(dispStr, "\ndataStart %lx\ndataEnd %lx\nblkStart %lx\nblkEnd %lx\nreadLoc %lx\nwriteError %d\nreadError %d\nCurrent offset is %d\n",
	    b->dataStart, b->dataEnd, b->blkStart, b->blkEnd, b->readLoc,
	    b->writeError, b->readError, (b->readLoc - b->blkStart));

	CS_LOG_PRINTS(IR_FILE, "SBuf Header\n");
	CS_LOG_PRINTS(IR_FILE, dispStr);
	CS_LOG_PRINTS(IR_FILE, "SBuf contents\n");
	CsCf_DumpBufData(b->blkStart, (SHORT)(b->blkEnd - b->blkStart));
}

/****************************************************************************
 * FUNCTION:  CsCf_CommErrorNo
 *
 * PURPOSE:
 *	This function provides a single interface for all communication
 *	functions wishing to obtain the error code from the linked in
 *	communication layer.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	TCP/IP error.
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

int CsCf_CommErrorNo()
{
	int errorCode;

#ifdef WINSOCKET
	errorCode = WSAGetLastError();
#endif
#ifdef BSDSOCKET
	errorCode = h_errno;
#endif
#ifdef WINPATHWAY
	errorCode = get_errno();
#endif

	return(errorCode);
}


/****************************************************************************
 * FUNCTION:  CsCf_ReportError
 *
 * PURPOSE:
 *	This function reports the error messages to the log file.
 *
 * INPUT:
 *	pointer to caller source
 *	error code
 *	pointer to assisting text
 *	additional data 
 *
 * OUTPUT:
 *	none.
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	CsCf_ReportError(CHAR *srcFile, SHORT code, CHAR *str, int data)
{
	CHAR	buf[100];

	sprintf(buf, "%d, %s, %d\n", code, str, data);
	CS_LOG_PRINTS(srcFile, buf);
}
