
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC Initialize 
 *
 * DESCRIPTION:
 *	This interface unit is used to initialize the communication
 *	service layer. Specifically it must be used to establish
 *	the mode of operation for the communication services. (ie,
 *	whether the services will support polled or event driven 
 *	configurations.
 *
 * CONTENTS:
 *	IRC_Initialize
 *
 * NOTES:
 *	This is a stub file.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <ir_cs.h>

extern BOOLEAN	RescUT_LogFlag;	
BOOLEAN	RescUT_LogAPDUs;

static BOOLEAN		ircInitialized=FALSE;

/****************************************************************************
 * FUNCTION: IRC_Initialize
 *
 * PURPOSE:
 *
 * INPUT:
 *
 *	There is no input required to this function.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *	This function is used to initialize the IR Comms layer.
 *
 ***************************************************************************/

IR_STATUS	IRC_Initialize( IRC_INIT_REQ *initReq )
{
	/* Check to see if already initialized. */
	if (CsCf_IsIRCInitialized())
	{
		CS_LOG_PRINTS(IR_FILE, "IRC Already initialized\n");
		return (IR_FAIL);
	}

	CS_LOG_OPEN;

	/* Check for initializing of log file */
	if (initReq->debugLog == TRUE)
	{
		RescUT_LogFlag = TRUE;
		CS_LOG_PRINTS(IR_FILE, "Initialized comms\n");
#ifdef WINSOCKET
		{
			extern WSADATA	WSAData;
			CS_LOG_PRINTV(IR_FILE, "Winsock dll version = %x\n",
		    	    WSAData.wVersion);
		}
#endif
	}
	else
	{
		CS_LOG_TRACES(IR_FILE, "Debugger not running\n");
		RescUT_LogFlag = FALSE;
	}

	/* Create the association tables */
	CsCf_AssocInit( initReq );


	/* Initialize buffers */
	CsCf_BuffInit();

	/* Set IRC comms to initialized */
	ircInitialized = TRUE;

#ifdef WINPATHWAY
	socket_init();
#endif

	/* Initialize the debug printing of apdus */
	RescUT_LogAPDUs = initReq->logAPDUs; 

	return IR_SUCCESS;
}

/****************************************************************************
 * FUNCTION: CsCf_IsIRCInitialized
 *
 * PURPOSE:
 *	This function is used to detect whether or not the IRC interface
 *	has been initialized.
 *
 * INPUT:
 *
 *	There is no input required to this function.
 *
 * OUTPUT:
 *	BOOLEAN			True - IRC has been initialized.
 *
 *
 * NOTES:
 *	This is a stub function.
 *
 ***************************************************************************/

BOOLEAN CsCf_IsIRCInitialized()
{
	return(ircInitialized);
}
