
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC P Data
 *
 * DESCRIPTION:
 *	This interface unit is used to send PDUs for IR data structures
 *	to a remote target.
 *
 * CONTENTS:
 *	IRC_P_Data
 *
 * NOTES:
 *	This is a stub file.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <resc.h>
#include <ir_cs.h>

#define BIG_BUF_SIZE	0x8000
static CHAR	bigBuf[BIG_BUF_SIZE];

static void chainSendData(CS_SOCK_ITEM *, SHORT , SHORT, CHAR *);
/****************************************************************************
 * FUNCTION: IRC_P_Data
 *
 * PURPOSE:
 *
 * INPUT:
 *	IRC Pdata structure pointer
 *	assocId to send data to
 *
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS	IRC_P_Data( IR_ASSOC_ID assocId, IRC_P_DATA_SEND *pData )
{
	CS_SOCK_ITEM	*sockItem;
	int		length;
	int		numSent;
	SBuf		apduBuf;

	if (!CsCf_IsIRCInitialized())
	{
		CS_LOG_PRINTS(IR_FILE, "IRC Not initialized\n");
		return(IR_FAIL);
	}

	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		CS_LOG_PRINTV(IR_FILE, "Association unknown: %d\n", assocId);
		return (IR_FAIL);
	}
	if (sockItem->state != CS_SOCK_ST_OPER)
	{
		CS_LOG_PRINTS(IR_FILE, "Cannot Send to a socket that is not connected\n");
		return(IR_FAIL);
	}

	/*
 	 * print the apdu first.
	 */
	CsCf_DisplayAPDU(pData->apdu);

	/* 
	 * Convert the p-data to a big buffer and send it.
	 */
	SBufInit(&apduBuf, bigBuf, BIG_BUF_SIZE);
	SBufResetInWriteRvsMode(&apduBuf);

	length = (int)BEncAPDU(&apduBuf, (APDU *)pData->apdu);
CS_LOG_PRINTS(IR_FILE, "Encoded the apdu into a byte stream\n");

	numSent = send (sockItem->socketFd, 
	    (char *)apduBuf.dataStart,
	    length, 0);

	if (numSent == SOCKET_ERROR)
	{
		CsCf_ReportError(IR_FILE, CS_ERR_IRC_SEND_ERROR, 
		    "Error sending pdata via socket",
		    CsCf_CommErrorNo());
		return(IR_FAIL);
	}

	if (numSent != length)
	{
		/* Pack remaining data into a list of send buffers. */
		chainSendData(sockItem, numSent, length, 
		    (char *)apduBuf.dataStart);
	}
	CS_LOG_PRINTV(IR_FILE, "Sent %d bytes ", numSent);
	CS_LOG_PRINTV("", "on %d association\n", sockItem->assocId);
	return(IR_SUCCESS);
}
void chainSendData(CS_SOCK_ITEM *sockItem, SHORT numSent, SHORT total,
    CHAR *srcBuf)
{
	SHORT	copySize;
	SHORT	srcStart;
	SHORT	totalSize;
	SHORT	errorCode;
	CS_BUFFER	*newBuf;

	totalSize = total - numSent;
	srcStart = numSent;

	while (totalSize > 0)
	{
		/* Get new buffer */
		newBuf = CsCf_BuffAlloc();

		/* Determine num bytes to copy */
		if (totalSize > newBuf->bufMallocSize)
		{
			copySize = newBuf->bufMallocSize;
		}
		else
		{
			copySize = totalSize;
		}
		/* Adjust total size remaining */
		totalSize = totalSize - copySize;


		/* Copy input data to buffer */
		memcpy(newBuf->buf, &srcBuf[srcStart], copySize);
		newBuf->bufSize = copySize;
		newBuf->bufState = CS_BUF_ST_SEND_DATA;

		srcStart = srcStart + copySize;

		/* Add the buffer to the send list */
		RescList_Append(IR_FILE, &sockItem->sendList, newBuf,
		    NULL, NULL);

	}

	if (RESC_LIST_SIZE(&sockItem->sendList) > 0)
	{
		/* Prime the socket to send data */
#ifdef WINSOCKET
		/* Add WRITE to select option */
		if (WSAAsyncSelect((SOCKET)sockItem->socketFd,
		    (HWND)sockItem->assocId,
		    (unsigned int)CS_SOCK_EV_RWC,
		    (long)(FD_READ|FD_CLOSE|FD_WRITE)) != 0)
		{
			errorCode = WSAGetLastError();
			CS_LOG_PRINTV(IR_FILE, 
			   "WSAAsynSelect Error. Error = %d\n",
			   errorCode);
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(sockItem->assocId, 
			    CS_SOCK_EV_ERROR_ABORT);
		}
#else
		/* Send send event */
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_SEND);
#endif
	}
}
