
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: CF_Recieve
 *
 * DESCRIPTION:
 *	This unit is used to convert stream data to an APDU.
 *
 * CONTENTS:
 *	CsCf_StreamToApdu
 *	CsCf_RecvAPDU
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#ifdef WINSOCKET
#include <stdlib.h>
#endif
#include <string.h>
#include <memory.h>
#include <ir_cs.h>
#include <ir_net.h>
#include <resc.h>

static CHAR	*decodeBuf;

static void bufChainToSBuf(IR_LIST *, SBuf *);

/****************************************************************************
 * FUNCTION: CsCf_StreamToApdu
 *
 * PURPOSE:
 *	This function will use the input assocId to grab the stream data
 *	from the association table and then convert that data to an
 *	apdu.
 *
 * INPUT:
 *	assocId
 *
 *
 * OUTPUT:
 *	APDU		
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
IR_STATUS CsCf_StreamToApdu(IR_LIST *buffList, APDU **apdu)
/*
 * This function will return a fail status if it could not
 * convert the stream data to an APDU.
 */
{
	APDU			*irApdu;
	AsnLen			decodedLen;
	int			val;
	CHAR			*buf;
	jmp_buf			env;	
	CS_BUFFER		*tmpBuf;
	SHORT			tmpCnt;
	SBuf			apduBuf;
	SHORT			i;

	if (RESC_LIST_SIZE(buffList) == 0)
	{
		CS_LOG_PRINTS(IR_FILE, "Cannot decode with empty buffers\n");
		return(IR_FAIL);
	}
	RESC_LIST_SET_FIRST(buffList);
	tmpBuf = (CS_BUFFER *)RESC_LIST_GET(buffList);
	/*
	 * Perform a little sanity checking. 
	 */
	if (!(tmpBuf->bufState & CS_BUF_ST_START_APDU))
	{
		CS_LOG_PRINTS(IR_FILE, "Trying to Decode from non-start buffer.\n");
		return (IR_FAIL);
	}

	/* Convert a possible buffer chain to an SBUF */
	bufChainToSBuf(buffList, &apduBuf);

	/* Reset the External Ok flag before decoding. */
	CsCf_IRExtReset();

	/* Decode the buffer */
	decodedLen = 0;
	if ((val = setjmp(env)) == 0)
	{
		irApdu = (APDU *)RescIR_Malloc(IR_FILE, sizeof(APDU));

		/* Convert the buffer to an APDU. */
        	BDecAPDU( &apduBuf, irApdu, &decodedLen, env);
		CS_LOG_PRINTV(IR_FILE, "Decoded %d bytes\n", decodedLen);
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "- ERROR Decode of buffer failed. %d\n",
		    val);
		/* Dump the apdu to a the log file */
		CsCf_DumpSBuf(IR_FILE, &apduBuf);
	}

	if (decodedLen > 0)
	{
		*apdu = irApdu;
	}
	else
	{
		RescIR_Free(IR_FILE, (CHAR *)irApdu);
	}

	while (RESC_LIST_SIZE(buffList) > 0)
	{
		RESC_LIST_SET_FIRST(buffList);
		tmpBuf = (CS_BUFFER *)RESC_LIST_GET(buffList);
		CsCf_BuffFree(tmpBuf);
		RescList_Delete(IR_FILE, buffList, 0);
	}
	if (decodeBuf != (CHAR *)NULL)
	{
		RescIR_Free(IR_FILE, decodeBuf);
		decodeBuf = (CHAR *)NULL;
	}

	if (!CsCf_IRExtIsOk())
	{
		/* Decoded an unknown external */
		RescIR_Free(IR_FILE, (CHAR *)irApdu);
		return(IR_FAIL);
	}


	return(IR_SUCCESS);
}


/****************************************************************************
 * FUNCTION: CsCf_RecvADPU
 *
 * PURPOSE:
 *	This function will convert a byte stream into an APDU.
 *
 * INPUT:
 *	IR_ASSOC_ID		- association receiving apdu.
 *
 * OUTPUT:
 *	IRC_COMM_EVENT_REPLY	- comm reply.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
IR_STATUS CsCf_RecvAPDU(IR_ASSOC_ID assocId, IRC_COMM_EVENT_REPLY *reply)
{
	CS_SOCK_ITEM	*sockItem;
	IR_STATUS	retStatus;
	CS_BUFFER	*buffer;
	CS_BUFFER	*tmpBuf;
	static BOOLEAN		writeFlag=FALSE;

CS_LOG_TRACES(IR_FILE, "In CsCf_RecvAPDU\n");

	retStatus = SUCCESS;
	sockItem = CsCf_AssocGet(assocId);
	if (!sockItem)
	{
		reply->replyType = IR_COMM_DATA_ERROR;
		retStatus = IR_FAIL;
		return(retStatus);
	}

	RESC_LIST_SET_FIRST(&sockItem->readList);
	buffer = (CS_BUFFER *)RESC_LIST_GET(&sockItem->readList);

	if (buffer == (CS_BUFFER *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Buffer missing\n");
		retStatus = IR_FAIL;
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_ERROR_ABORT);
		reply->replyType = IR_COMM_DATA_NO_DATA;
		return(retStatus);
	}

	/*
	 * Send first buffer of list of 1 or more to be decoded.
	 */
	if (CsCf_StreamToApdu(&sockItem->readList, &reply->apdu) == IR_FAIL)
	{
		retStatus = IR_FAIL;
		CS_LOG_PRINTS(IR_FILE, "Stream Conversion Failed\n");
		sockItem->state = CS_SOCK_ST_ABORT;
		CsCf_Notify(sockItem->assocId, CS_SOCK_EV_ERROR_ABORT);
		reply->replyType = IR_COMM_DATA_NO_DATA;
	}
	else
	{
		reply->replyType = IR_COMM_DATA_APDU;
	}

CS_LOG_TRACES(IR_FILE, "Exiting CsCf_RecvAPDU\n");
	return(retStatus);
}

/*
 * This is a debug function only. It is not available in a 
 * production mode.
 */
void CsCf_RecvIntoSaveFile(CHAR *buf, USHORT sizeOfAPDU, APDU *irApdu, SHORT sOrR)
{
	FILE	*fp;
	CHAR	fileName[20];

	switch(irApdu->choiceId)
	{
	case APDU_INITRESP:
	case APDU_INITREQ:
		if (sOrR > 0)
		{
			strcpy(fileName, "initResp.tg");
		}
		else
		{
			strcpy(fileName, "initReq.tg");
		}
		break;
	case APDU_SEARCHRESP:
	case APDU_SEARCHREQ:
		if (sOrR > 0)
		{
			strcpy(fileName, "srchResp.tg");
		}
		else
		{
			strcpy(fileName, "srchReq.tg");
		}
		break;
	case APDU_PRESENTRESP:
	case APDU_PRESENTREQ:
		if (sOrR > 0)
		{
			strcpy(fileName, "presentResp.tg");
		}
		else
		{
			strcpy(fileName, "presentReq.tg");
		}
		break;
	default:
		if (sOrR > 0)
		{
			strcpy(fileName, "target.tg");
		}
		else
		{
			strcpy(fileName, "origin.tg");
		}
		break;
	}

	if (strlen(fileName) > 0 )
	{
		fp = fopen(fileName, "w+");
		fwrite(buf, 1, sizeOfAPDU, fp);
		fflush(fp);
		fclose(fp);
	}
} 

void bufChainToSBuf(IR_LIST *chainList, SBuf *sBuf)
{
	CS_BUFFER		*dataBuf;
	APDU			*irApdu;
	AsnLen			decodedLen;
	int			val;
	CHAR			*buf;
	CS_BUFFER		*tmpBuf;
	SHORT			tmpCnt;
	SHORT			i;

	/* Initialize the decode buffer to null */
	decodeBuf = (CHAR *)NULL;

	RESC_LIST_SET_FIRST(chainList);
	dataBuf = (CS_BUFFER *)RESC_LIST_GET(chainList);

	/* Check for entire APDU in a single buffer */
	if (dataBuf->bufSize == dataBuf->sizeOfAPDU)
	{
		SBufInstallData(sBuf, dataBuf->buf, dataBuf->sizeOfAPDU);
		return;
	}

	/* Create a decode buf large enough to hold the data */
	decodeBuf = (CHAR *)RescIR_Malloc(IR_FILE, (ULONG)dataBuf->sizeOfAPDU);
		
	/* Copy all of chain into decodeBuf */
	RESC_LIST_SET_FIRST(chainList);
	i = 0;
	tmpCnt = 0;
	while (i < RESC_LIST_SIZE(chainList))
	{
		tmpBuf = (CS_BUFFER *)RESC_LIST_GET(chainList);

		memcpy(&decodeBuf[tmpCnt], tmpBuf->buf, tmpBuf->bufSize);

		tmpCnt += tmpBuf->bufSize;

		/* Copy next buffer into decode buffer */
		RESC_LIST_SET_NEXT(chainList);
		i++;
	}

	SBufInstallData(sBuf, decodeBuf, dataBuf->sizeOfAPDU);
}
