
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IRC_Connect in resolve.c
 *
 * DESCRIPTION:
 *	This interface unit is used to initiate a connection on a 
 *	TCP/IP network using either the host name or the address.
 *
 * CONTENTS:
 *	IRC_Connect
 *
 * NOTES:
 *	This is a stub file.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#ifdef WINSOCKET
#include <stdlib.h>
#endif
#include <memory.h>
#include <ir_pli.h>
#include <ir_net.h>
#include <ir_cs.h>
#include <resc.h>
#ifdef BSDSOCKET
#include <fcntl.h>
#endif


/****************************************************************************
 * FUNCTION: IRC_Connect
 *
 * PURPOSE:
 *	This function will initiate the a connect to a remote TCP/IP host
 *	using either the address or host name as the address.
 *
 * INPUT:
 *
 *
 * OUTPUT:
 *	This function returns an IR_STATUS which returns a non-zero
 *	value on error. 
 *
 *
 * NOTES:
 *	This function will invoke the proper WINSOCKET or BSDSOCKET 
 *	connect function. The winConnect and bsdConnect functions are
 *	covered by this header.
 *
 ***************************************************************************/

IR_STATUS	IRC_Connect( IRC_ASSOC_REQ *request )
{
	HANDLE				handle;
	CS_SOCK_ITEM			*sockItem;
	SHORT				errorCode;
	HOSTENT				*hostp;
	HOSTENT				*hostp2;
	long				addr;
	CHAR				tmpBuf[IR_LEN_100];

	if (!CsCf_IsIRCInitialized())
	{
		return(IR_FAIL);
	}

#ifdef WINSOCKET
	if ((hostp = (HOSTENT *)RescIR_Zmalloc(
	    IR_FILE,
	    MAXGETHOSTSTRUCT)) == (HOSTENT *)NULL)
#else
	if ((hostp = (HOSTENT *)RescIR_Zmalloc(
	    IR_FILE,
	    sizeof(HOSTENT))) == (HOSTENT *)NULL)
#endif
	
	{
		CS_LOG_PRINTS(IR_FILE, "Memory error detected\n");
		return(IR_FAIL);
	}


	/* create the new socket item */
	sockItem = CsCf_AssocNew(request->assocId);
	sockItem->port = request->portNum;

	CS_LOG_PRINTV(IR_FILE, "Connecting assocId %d\n", request->assocId);


	/*
 	 * Use synchronous calls to allow compatibility with chameleon.
 	 */
	if (request->hostName.len > 0)
	{
		VSToStrNullTerm(tmpBuf, &request->hostName, IR_LEN_100);
		CS_LOG_PRINTV(IR_FILE, "HostName %s\n", tmpBuf);
			   
		if ((hostp2 = gethostbyname(tmpBuf)) == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "gethostbyname failed\n");
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(request->assocId, CS_SOCK_EV_FAIL_RESOLVE);
			sockItem->hostp = hostp;
			return(IR_SUCCESS);
		}
		else
		{
			CS_LOG_PRINTV(IR_FILE, "Found hostname for %s\n", 
			    tmpBuf);

			(void)memcpy((CHAR *)hostp,
			    (CHAR *)hostp2, 
			    sizeof(struct hostent));

			/* Add new host to assoc list. */
			sockItem->hostp = hostp;
			sockItem->state = CS_SOCK_ST_RESOLVE;
			CsCf_Notify(request->assocId, CS_SOCK_EV_RESOLVED);
			return(IR_SUCCESS);
		}	
	}
	else if (request->hostAddr.len > 0)
	{
		VSToStrNullTerm(tmpBuf, &request->hostAddr, 200);
		/* 
		 * Convert the dotted string format to ip network
		 * format.
		 */
		CS_LOG_PRINTV(IR_FILE, "hostAddress = %s\n", tmpBuf);
		addr = inet_addr(tmpBuf);
		if ((hostp2 = gethostbyaddr( &addr, 4, PF_INET)) == NULL)
		{
			CS_LOG_PRINTS(IR_FILE, "gethostbyaddr failed\n");
			sockItem->state = CS_SOCK_ST_ABORT;
			CsCf_Notify(request->assocId, CS_SOCK_EV_FAIL_RESOLVE);
			sockItem->hostp = hostp;
			return(IR_SUCCESS);
		}
		else
		{
			(void)memcpy((CHAR *)hostp,
			    (CHAR *)hostp2, 
			    sizeof(struct hostent));

			/* Add new host to assoc list. */
			sockItem->hostp = hostp;
			sockItem->state = CS_SOCK_ST_RESOLVE;
			CsCf_Notify(request->assocId, CS_SOCK_EV_RESOLVED);
			return(IR_SUCCESS);
		}
	}

	CS_LOG_PRINTS(IR_FILE, "Unknown hostname or address\n");
	return(IR_FAIL);
}
