
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Decode External
 *
 * DESCRIPTION:
 *	This unit contains functions to interpret external object identifiers
 *	and provide the proper single-ASN1-type decode/encode/print/free
 *	functions. This unit also contains functions used to perform the
 *	decode/encode/print/free functions.
 *
 * CONTENTS:
 *	CsEd_ExtCheck
 *
 * NOTES:
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
/***************************************************************************/

#include <ir.h>
#include <ir_pli.h>
#include <apdu.h>
#include <ir_cs.h>
#include <ir_oid.h>
#include <debug.h>
#include <report.h>


/* 
 * This array of character pointers provides a list of all
 * object identifiers known by this communication services.
 * The object identifiers are supplied as a null terminated
 * array of null terminated dotted strings.
 */
typedef struct
{
	CHAR	*oid;
	void	(*decodeFn)();
	void 	(*encodeFn)();
	void	(*printFn)();
	void	(*freeFn)();
} OID_TO_FUNC;


void CsEd_DecRsrcRprtFmt ( BUF_TYPE b, EXTERNALChoice *v, 
    AsnLen *bytesDecoded, ENV_TYPE env);
void CsEd_PrintRsrcRprtFmt (FILE *f, EXTERNALChoice *v,  USHORT indent);
void CsEd_FreeRsrcRprtFmt ( EXTERNALChoice *v) ;

static OID_TO_FUNC extOidsToFunc[]=
{
	{
		OID_RESOURCE_REPORT_FMT_BIB1,
		CsEd_DecRsrcRprtFmt,
		NULL, 
		CsEd_PrintRsrcRprtFmt, 
		CsEd_FreeRsrcRprtFmt	
	},
	{
		OID_ATTRIBUTE_SET_BIB1,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_DIAGNOSTIC_SET_BIB1,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_UNIMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_INTERMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_CCF ,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_USMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_UKMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_NORMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_LIBRISMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_DANMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_FINMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_MAB,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_CANMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_SBN,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_REC_SYN_PICAMARC,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{

		OID_ACCESS_CTRL_SKL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_RESOURCE_REPORT_FMT_SKL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		OID_IR_UNKNOWN_EXTERNAL_SKL,
		NULL,
		NULL,
		NULL,
		NULL
	},
	{
		(CHAR *)NULL,
		NULL, 
		NULL, 
		NULL, 
		NULL
	}
};

static BOOLEAN	CSCF_IRExtOk=TRUE;

/* 
 * This function checks for known externals. It is used to determine the
 * appropriate processing function for single-ASN1-type.
 * Any external without an object identifier will be
 * rejected causing the association to terminate.
 */
IR_STATUS CsEd_ExtCheck(EXTERNAL *v, void (**fn)(), SHORT funcType)
{

	CHAR			oidString[100];
	OBJECT_IDENTIFIER 	oid;
	SHORT			i;


	RESC_ASN_NO_MALLOC(&oid);
	RESC_ASN_INIT_OI(&oid);

	/* Prepare return func in case of error */
	*fn = (void (*)())NULL;

	/* Check for object identifier in EXTERNAL */
	if (!ASNOCTS_PRESENT(&v->direct_reference))
	{
		/* No direct_reference. This is a bad external. */
		CS_LOG_PRINTS(IR_FILE, "Missing Object Identifier in external\n");
		return IR_FAIL;
	}

	/* Check object identifier against all known identifiers. */
	RescASN_SnaccToIROI(IR_FILE, &oid, &v->direct_reference);
	VSToStrNullTerm(oidString, &oid, 100);
	i = 0;
	while (extOidsToFunc[i].oid != (CHAR *)NULL)
	{
		if (strcmp(oidString, extOidsToFunc[i].oid) == 0)
		{
			/* Found a match */
			switch(funcType)
			{
			case IR_ED_DECODE_FN:
				*fn = extOidsToFunc[i].decodeFn;
				break;
			case IR_ED_ENCODE_FN:
				*fn = extOidsToFunc[i].encodeFn;
				break;
			case IR_ED_PRINT_FN:
				*fn = extOidsToFunc[i].printFn;
				break;
			case IR_ED_FREE_FN:
				*fn = extOidsToFunc[i].freeFn;
				break;
			default:
				CS_LOG_PRINTV(IR_FILE, 
				    "Unknown funcType %d in external check\n",
				    funcType);
			}

			RescASN_DestroyOI(IR_FILE, &oid);
			return IR_SUCCESS;
		}
		i++;
	}
	RescASN_DestroyOI(IR_FILE, &oid);

	CsCf_BadExternal();
	return IR_FAIL;
	
}

void CsEd_DecRsrcRprtFmt (BUF_TYPE b, EXTERNALChoice *v, 
    AsnLen *bytesDecoded, ENV_TYPE env)
{
	RsrcReport *newRsrcRprt;

	newRsrcRprt = (RsrcReport *)Asn1Alloc(sizeof(RsrcReport));

	memset(newRsrcRprt, (CHAR)NULL, sizeof(RsrcReport));

	BDecRsrcReport ( b, newRsrcRprt, bytesDecoded, env);
}

void CsEd_PrintRsrcRprtFmt (FILE *f, EXTERNALChoice *v, USHORT indent) 
{
	if ((v->a.single_ASN1_type) != (AsnOcts *)NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Printing rsrcReportFmt\n");
		PrintRsrcReport(f, (RsrcReport*)v->a.single_ASN1_type, 
		    indent);
	}
}

void CsEd_FreeRsrcRprtFmt ( EXTERNALChoice *v)
{
	if ((v->a.single_ASN1_type) != (AsnOcts *)NULL)
	{
		FreeRsrcReport ((RsrcReport*) v->a.single_ASN1_type);
	}
}
