
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

 /***************************************************************************
 * UNIT: State Table Operations
 *
 * DESCRIPTION: This unit contains the functions required for creating,
 *	deleting, checking and updating an association's state table.
 *
 * CONTENTS: OkAse_CheckEvent
 *	     OkAse_UpdateState
 *	     OkAse_CreateAssoc
 *	     OkAse_CheckAssoc
 *	     OkAse_DeleteAssoc
 *	     OkAse_DeleteAllAssocs
 *	     OkAse_InitAssoc
 *
 * NOTES:
 *	Nothing of Note.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
***************************************************************************/
#include <stdio.h>
#include <ir.h>
#include <ir_ali.h>
#include <resc.h>
#include <ok/ok.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OKIRASE statetbl.c"


ASSOC_DATA_TYPE *Ase_assocListPtr;
static ASSOC_DATA_TYPE *lastAssocListPtr;

static SHORT STATE_TABLE [MAX_EVENTS][MAX_STATES] =
	{ { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 3, 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 10, 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 4, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 3, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 5, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 3, 0, 0, 0, 0, 0 },
	{ 0, 0, 6, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 3, 0, 0, 0, 0 },
	{ 0, 0, 7, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 3, 0, 0, 0 },
	{ 0, 2, 0, 4, 5, 6, 0, 0, 0, 0 },
	{ 0, -8, 0, -8, -8, -8, 0, 0, 0, 0 },
	{ 0, 2, 0, 4, 5, 6, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, POPST, 0, 0 },
	{ 0, -9, 0, -9, -9, -9, -9, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, 0, POPST, 0 },
	{ 0, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
	{ 0, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
	{ 0, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
	{ 0, 0, 10, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 } };

static SHORT ALL_EVENTS [MAX_EVENTS] =
	{ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
/****************************************************************************
 * FUNCTION: OkAse_CreateAssoc
 *
 * PURPOSE: Create a list of accepted events for the new association,
 *		initialize currentState and previousState. 
 *		Add the assocation to the list of current associations.
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *
 * OUTPUT: BOOLEAN	IR_SUCCESS if success, IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_CreateAssoc(
    IR_ASSOC_ID assocId)
{
	ASSOC_DATA_TYPE	*newAssocPtr;
	SHORT		i;

	/* Create new association structure. */
	newAssocPtr = (ASSOC_DATA_TYPE *)RescIR_Malloc( 
	    IR_FILE, sizeof(ASSOC_DATA_TYPE));
	if (newAssocPtr == NULL)
	{
		/* Error - unable to allocate memory */
		OkCf_UpdateErrorReport(IR_ERROR_FATAL, IRK_MEMORY_ERROR, 
		    IR_FILE);
		return (IR_FAIL);
	}

	if (Ase_assocListPtr == NULL)
	{
		Ase_assocListPtr = newAssocPtr;
	}
	else
	{
		lastAssocListPtr->nextAssocPtr = newAssocPtr;
	}
	newAssocPtr->assocId = assocId;
	lastAssocListPtr = newAssocPtr;

	/* Initialize association structure. */
	for (i = 0; i < MAX_EVENTS; i++) 
	{
		newAssocPtr->validEvents[i] = ALL_EVENTS[i];
	}
	newAssocPtr->currentState = OK_CLOSED;
	newAssocPtr->previousState = OK_INVALID;
	newAssocPtr->nextAssocPtr = NULL;
	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAse_CheckAssoc
 *
 * PURPOSE: Check if the association already exists
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *
 * OUTPUT: BOOLEAN	IR_SUCCESS if association exists,
 *			IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_CheckAssoc(
    IR_ASSOC_ID assocId)
{
	ASSOC_DATA_TYPE	*prevAssocPtr;
	ASSOC_DATA_TYPE	*assocPtr;

	/* Verify that association does not already exist. */
	prevAssocPtr = NULL;
	assocPtr = Ase_assocListPtr;
	while (assocPtr != NULL && assocPtr->assocId != assocId)
	{
		prevAssocPtr = assocPtr;
		assocPtr = assocPtr->nextAssocPtr;
	}	
	if (assocPtr == NULL)
	{
		return (IR_FAIL);
	}
	
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAse_DeleteAssoc
 *
 * PURPOSE: Delete an association from the list of current associations.
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *
 * OUTPUT: BOOLEAN 	IR_SUCCESS if success, IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_DeleteAssoc(
    IR_ASSOC_ID assocId)
{
	ASSOC_DATA_TYPE *assocPtr;
	ASSOC_DATA_TYPE	*prevAssocPtr;
	ASSOC_DATA_TYPE	*nextAssocPtr;

	/* Find association. */
	assocPtr = Ase_assocListPtr;
	prevAssocPtr = NULL;
	while (assocPtr != NULL && assocPtr->assocId != assocId)
	{
		prevAssocPtr = assocPtr;
		assocPtr = assocPtr->nextAssocPtr;
	}	

 	if (assocPtr == NULL) 
	{
		/* Error - invalid association */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* Delete association structure. */
	if (prevAssocPtr == NULL)
	{
		Ase_assocListPtr = assocPtr->nextAssocPtr;
		assocPtr = (ASSOC_DATA_TYPE *)RescIR_Free( 
	            IR_FILE, (CHAR *)assocPtr);
	}
	else
	{
		nextAssocPtr = assocPtr->nextAssocPtr;
		assocPtr = (ASSOC_DATA_TYPE *)RescIR_Free( 
	            IR_FILE, (CHAR *)assocPtr);
		prevAssocPtr->nextAssocPtr = nextAssocPtr;
	}

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAse_DeleteAllAssocs
 *
 * PURPOSE: Delete the list of associations.
 *
 * INPUT:
 *
 * OUTPUT: BOOLEAN 	IR_SUCCESS if success, IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_DeleteAllAssocs()
{
	IR_STATUS	status;
	ASSOC_DATA_TYPE *assocPtr;

	while (Ase_assocListPtr != NULL)
	{
		assocPtr = Ase_assocListPtr;
		status = OkAse_DeleteAssoc(assocPtr->assocId);
		if (status == IR_FAIL)
		{
			return (IR_FAIL);
		}
	}	

	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAse_CheckEvent
 *
 * PURPOSE: Check if the event if a valid event for the association
 *	specified and the current state of that association.
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *	  SHORT		event identifier
 *
 * OUTPUT: BOOLEAN	IR_SUCCESS if event is valid, IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_CheckEvent(
    IR_ASSOC_ID	assocId,
    SHORT	event)
{
	ASSOC_DATA_TYPE *assocPtr;

	/* Find association. */
	assocPtr = Ase_assocListPtr;
	while (assocPtr != NULL && assocPtr->assocId != assocId)
	{
		assocPtr = assocPtr->nextAssocPtr;
	}	
 	if (assocPtr == NULL) 
	{
		/* Error - invalid association */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* 
	 * Return IR_SUCCESS if and only if the event is serviced by
	 *  the association and the event is valid for the current 
	 *  state of the association. 
	 */
	if ((event > MAX_EVENTS) || (event <= 0))
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_INVALID_EVENT, IR_FILE);
		return (IR_FAIL);
	}

	if (assocPtr->validEvents [event - 1] != 1) 
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_EVENT_NOT_SUPPORTED, IR_FILE);
		return (IR_FAIL);
	}
	if (STATE_TABLE [event - 1][assocPtr->currentState - 1] == 0)
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_INVALID_EVENT_FOR_STATE, IR_FILE);
		return (IR_FAIL);
	}
	return (IR_SUCCESS);	
}

/****************************************************************************
 * FUNCTION: OkAse_UpdateState
 *
 * PURPOSE: Given the event and current state of the specified association,
 *	determine the next state and update the current state appropriately.
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *	  SHORT		event identifier
 *
 * OUTPUT: BOOLEAN	IR_SUCCESS if state successfully updated, IR_FAIL otherwise
 *
 * NOTES: Values for the state table are as follows:
 *		< 0 -> previousState = currentState
 *	 	       currentState = abs [value]
 *
 *		= 0 -> invalid event 
 *	 	> 0, < maxStates -> currentState = value
 *		> maxStates -> some action
 *				only currently defined action is POPST 
 * 					-> currentState = previousState
 *
 ***************************************************************************/
IR_STATUS OkAse_UpdateState(
    IR_ASSOC_ID	assocId,
    SHORT	event)
{
	ASSOC_DATA_TYPE *assocPtr;
	SHORT		nextState;

	/* Find association. */
	assocPtr = Ase_assocListPtr;
	while (assocPtr != NULL && assocPtr->assocId != assocId)
	{
		assocPtr = assocPtr->nextAssocPtr;
	}	
 	if (assocPtr == NULL) 
	{
		/* Error - invalid association */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}
	if ((event > MAX_EVENTS) || (event <= 0))
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_INVALID_EVENT, IR_FILE);
		return (IR_FAIL);
	}
	if (assocPtr->validEvents [event - 1] != 1) 
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_EVENT_NOT_SUPPORTED, IR_FILE);
		return (IR_FAIL);
	}
	nextState = STATE_TABLE[event - 1][assocPtr->currentState - 1];
	if (nextState == 0)
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_INVALID_EVENT_FOR_STATE, IR_FILE);
		return (IR_FAIL);
	}
	else if (nextState > MAX_STATES)
	{
		if (nextState == POPST)
		{
			assocPtr->currentState = assocPtr->previousState;
			assocPtr->previousState = 0;
		}
		else
		{
			/* Error - invalid event */
			OkCf_UpdateErrorReport(IR_ERROR_WARNING,
			    IRK_INVALID_EVENT, IR_FILE);
			return (IR_FAIL);
		}
	}
	else if (nextState > 0) /* and =< MAX_STATES */
	{
		assocPtr->currentState = nextState;
	}
	else if (nextState >= -(MAX_STATES)) /* and < 0 */
	{
		assocPtr->previousState = assocPtr->currentState;
		assocPtr->currentState = abs(nextState);
	}
	else /* nextState < -MAX_STATES */
	{
		/* Error - invalid event */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_INVALID_EVENT, IR_FILE);
		return (IR_FAIL);
	}
	return (IR_SUCCESS);
}

/****************************************************************************
 * FUNCTION: OkAse_InitAssoc
 *
 * PURPOSE: Given the service options, update the list of valid events for
 * 	the association specified.
 *
 * INPUT: IR_ASSOC_ID	association identifier
 *	  BIT_STRING	service options
 *
 * OUTPUT: BOOLEAN	IR_SUCCESS if association successfully initiated, 
 *	IR_FAIL otherwise
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS  OkAse_InitAssoc(
    IR_ASSOC_ID	assocId,
    BIT_STRING	*services)
{
	ASSOC_DATA_TYPE *assocPtr;
	CHAR		currentByte;
	CHAR		*bitString;
	USHORT		numBits;

	numBits = services->len;
	bitString = services->sPtr;

	/* Find association. */
	assocPtr = Ase_assocListPtr;
	while (assocPtr != NULL && assocPtr->assocId != assocId)
	{
		assocPtr = assocPtr->nextAssocPtr;
	}	
 	if (assocPtr == NULL) 
	{
		/* Error - invalid association */
		OkCf_UpdateErrorReport(IR_ERROR_WARNING,
		    IRK_ASSOC_DOES_NOT_EXIST, IR_FILE);
		return (IR_FAIL);
	}

	/* If service is masked out, clear event. */
	currentByte = bitString[0];
	if ((currentByte & 0x80) == 0x00)
	{
		assocPtr->validEvents [3] = 0;
		assocPtr->validEvents [4] = 0;	
	}			
	if ((currentByte & 0x40) == 0x00)
	{
		assocPtr->validEvents [5] = 0;
		assocPtr->validEvents [6] = 0;	
	}			
	if ((currentByte & 0x20) == 0x00)
	{
		assocPtr->validEvents [7] = 0;
		assocPtr->validEvents [8] = 0;	
	}			
	if ((currentByte & 0x10) == 0x00)
	{
		assocPtr->validEvents [9] = 0;
		assocPtr->validEvents [10] = 0;	
	}			
	if ((currentByte & 0x08) == 0x00)
	{
		assocPtr->validEvents [11] = 0;
	}			
	if ((currentByte & 0x04) == 0x00)
	{
		assocPtr->validEvents [12] = 0;
		assocPtr->validEvents [13] = 0;	
		assocPtr->validEvents [14] = 0;	
	}			
	if ((currentByte & 0x02) == 0x00)
	{
		assocPtr->validEvents [15] = 0;
		assocPtr->validEvents [16] = 0;	
	}			
	
	return (IR_SUCCESS);
}
