
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OpCf_err
 *
 * DESCRIPTION:
 *	This unit contains functions to save and report errors.
 *
 * CONTENTS:
 *	OpCf_ErrorReport
 *	OpCf_UpdateErrors
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <string.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>

static IRO_ERROR_REPLY	currentError;

/****************************************************************************
 * FUNCTION: IRO_ErrorReport 
 *
 * PURPOSE: Return report of last error.
 *
 * INPUT: 
	IRO_ERROR_REPLY *error;
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS IRO_ErrorReport(
	IRO_ERROR_REPLY *error)
{
	error->errorLevel = currentError.errorLevel;
	error->errorCode = currentError.errorCode;
	(void)strncpy(error->errorString, currentError.errorString, 
		IRO_MAX_ERROR_STRING);
	error->errorString[IRO_MAX_ERROR_STRING] = EOS;

	return (IR_SUCCESS);
}
/****************************************************************************
 * FUNCTION: OpCf_UpdateErrorReport 
 *
 * PURPOSE: Update report to reflect the latest error.
 *
 * INPUT: 
 *	IR_ERROR_LEVEL level
 *	IR_ERROR_CODE  code
 *	CHAR	       *source
 *
 * OUTPUT: IR_STATUS
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/
IR_STATUS OpCf_UpdateErrorReport(
	IR_ERROR_LEVEL level,
	IR_ERROR_CODE  code,
	CHAR	       *source)
{
	CHAR		buf[100];
	IRK_ERROR_REPLY	irkErr;

	/* Copy the error codes to the static element */
	currentError.errorLevel = level;
	currentError.errorCode = code;


	if (source != NULL)
	{
		strcpy(buf, source);
		CS_LOG_PRINTV(IR_FILE, "Error detected from %s\n", buf);
		strcpy(currentError.errorString, source);
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "Error detected unknown source. code = %d\n", code);
		strcpy(currentError.errorString, "");
	}

	if (code == IRO_ALI_ERROR)
	{
		/* Get the ali error code, level and location */
			(void)IRK_ErrorReport(&irkErr);

		/* Override the local error with the ALI error */
		currentError.errorLevel = irkErr.errorLevel;
		currentError.errorCode = irkErr.errorCode;
		/* Concatenate the two error strings */
		strcat(currentError.errorString, irkErr.errorSource);
	}

	return (IR_SUCCESS);
}

void OpCf_ErrorInit()
{
	currentError.errorLevel = 0;
	currentError.errorCode = 0;
	strcpy(currentError.errorString, "");
}
