
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OPCF_Initialize
 *
 * DESCRIPTION:
 *	This interface CSU contains the functions needed to perform
 *	the initialization of the Origin Process.
 *
 * CONTENTS:
 *	IRO_Initialize
 *	OpCf_IsOpInitialize
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <op/ir_op.h>
#include <resc.h>

static BOOLEAN	opInitialized=FALSE;

/****************************************************************************
 * FUNCTION:  IRO_Initialize
 *
 * PURPOSE:
 *	This IRO interface function will initialize the origin process 
 *	then invoke the IRK_Initialize primitive to initialize the 
 *	origin kernel.
 *
 * INPUT:
 *	commConfig		- communications configuration
 *	languageConfig		- language configuration
 *	commStack		- TCP/IP or OSI (OSI not supported)
 *	
 *
 *
 * OUTPUT:
 *	This function returns an IR_STATUS.
 *
 *
 * NOTES:
 *	A call to IRO_ErrorReport is required to inform the 
 *	user of any problems.
 *
 ***************************************************************************/

IR_STATUS	IRO_Initialize( IRO_INIT_REQ *startupReq )
{
	IR_STATUS	status;
	IRK_INIT_REQ	irkInit;

	if (OpTm_Initialize() != SUCCESS)
	{
		CS_LOG_PRINTS(IR_FILE, "OpTm_Initialize failed\n");
		return(IR_FAIL);
	}
	status = OpAm_AssocInit();
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_MEMORY_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	OpWs_WsInit();

	irkInit.commConfig = startupReq->commConfig;
	irkInit.debugLog = startupReq->debugLog;
	irkInit.logAPDUs = startupReq->logAPDUs;
	status = IRK_Initialize(&irkInit);
	if (status == IR_FAIL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_ALI_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	OpCf_ErrorInit();
	opInitialized = TRUE;
	return IR_SUCCESS;
}

/****************************************************************************
 * FUNCTION:  OpCf_IsOpInitialized
 *
 * PURPOSE:
 *	This function is used by Origin Process CSCs to determine 
 *	whether or not the IRO interface has been initialized.
 *	This function can only be called once.
 *
 * INPUT:
 *	none.
 *
 * OUTPUT:
 *	Returns IR_STATUS. SUCCESS if IRO has been initialized.
 *
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/
BOOLEAN	OpCf_IsOpInitialized()
{
	return (opInitialized);
}
