/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: OPCF_Terminate
 *
 * DESCRIPTION:
 *	This interface CSU contains the functions needed to perform
 *	the termination of the Origin Process.
 *
 * CONTENTS:
 *	IRO_Terminate
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ir_ali.h>
#include <resc.h>
#include <op/ir_op.h>
#include <resclist.h>

#ifdef FILENAME
#undef FILENAME
#endif
#define FILENAME "OPCF cf_term.c"

/****************************************************************************
 * FUNCTION:  IRO_Terminate
 *
 * PURPOSE:
 *	This IRO interface function will terminate the origin process 
 *	then invoke the IRK_Terminate primitive to terminate the 
 *	origin kernel. This function can only be called once.
 *
 * INPUT:
 * 	none.	
 *
 * OUTPUT:
 *	This function returns an IR_STATUS.
 *
 *
 * NOTES:
 *	A call to IRO_ErrorReport is required to inform the 
 *	user of any problems.
 *
 ***************************************************************************/

IR_STATUS	IRO_Terminate ( )
{
	extern IR_LIST	*OpAm_AssocList;
	IR_STATUS	status;


	if (OpCf_IsOpInitialized())
	{
		if (RESC_LIST_SIZE(OpAm_AssocList) > 0)
		{
			/* Abort all active associations */
		}
		/* Terminate the IRK interface */
		status = IRK_Terminate();
		if (status == IR_FAIL)
		{
			OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_ALI_ERROR,
			    IR_FILE);
		}

		/* Destroy the associaiton list. */
		OpAm_AssocListFree();
		OpCf_IROFree();
	}

	return IR_SUCCESS;
}
