

/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: tm_init.c
 *
 * DESCRIPTION:
 *	This unit contains the functions used get system info
 *
 * CONTENTS:
 *	OpTm_InitReqInfo
 *	OpTm_InitReqInfoFree
 *	OpTm_Initialize
 *	OpTm_WorkPath
 *	OpTm_UserInfoFileName
 *	OpTm_UserInfoFree
 *
 * NOTES:
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/
#include <stdlib.h>
#include <ir.h>
#include <ir_api.h>
#include <resc.h>
#include <rescio.h>
#include <asn_tool.h>
#include <op/ir_op.h>


static VISIBLE_STRING 	implementationId;
static VISIBLE_STRING 	implementationVersion;
static VISIBLE_STRING 	implementationName;
static LONG 		preferredMessageSize;
static LONG 		maximumRecordSize;
static BIT_STRING 	options;
static BIT_STRING 	protocolVersion;
static VISIBLE_STRING	userInfoFileName;
static VISIBLE_STRING	workPath;


/****************************************************************************
 * FUNCTION:  OpTm_InitReqInfo
 *
 * PURPOSE:
 *	This function is used to return the association target information.
 *
 * INPUT:
 *	IRO_INIT_REQ_INFO	pointer to init req info
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *	The info pointer must be freed by the calling function. 
 *
 ***************************************************************************/

IR_STATUS	OpTm_InitReqInfo(IR_TARGET_ID targetId, OP_INIT_REQ_INFO *info)
{

	/* Copy implementation id */
	RESC_ASN_NO_MALLOC(&(info->implementationId));
	RESC_ASN_INIT_VS(&(info->implementationId));
	RescASN_CopyVS(IR_FILE, &(info->implementationId), &implementationId);

	/* Copy implementation name */
	RESC_ASN_NO_MALLOC(&(info->implementationName));
	RESC_ASN_INIT_VS(&(info->implementationName));
	RescASN_CopyVS(IR_FILE, &(info->implementationName), &implementationName);
	/* Copy implementation version */
	RESC_ASN_NO_MALLOC(&(info->implementationVersion));
	RESC_ASN_INIT_VS(&(info->implementationVersion));
	RescASN_CopyVS(IR_FILE, &(info->implementationVersion), 
	    &implementationVersion);

	/* Copy limits */
	info->preferredMessageSize = preferredMessageSize;
	info->maximumRecordSize = maximumRecordSize;

	/* Copy options */
	RESC_ASN_NO_MALLOC(&(info->options));
	RESC_ASN_INIT_BS(&(info->options));
	RescASN_CopyBS(IR_FILE, &(info->options), &options);

	/* Copy protocol version */
	RESC_ASN_NO_MALLOC(&(info->protocolVersion));
	RESC_ASN_INIT_BS(&(info->protocolVersion));
	RescASN_CopyBS(IR_FILE, &(info->protocolVersion), &protocolVersion);

	return (IR_SUCCESS);
	
}
 
void OpTm_InitReqInfoFree(OP_INIT_REQ_INFO *iri)
{
	if (iri)
	{
		RescASN_DestroyBS(IR_FILE, &iri->protocolVersion);
		RescASN_DestroyBS(IR_FILE, &iri->options);
		RescASN_DestroyVS(IR_FILE, &iri->implementationId);
		RescASN_DestroyVS(IR_FILE, &iri->implementationName);
		RescASN_DestroyVS(IR_FILE, &iri->implementationVersion);
		if (RESC_ASN_IS_MALLOCED(iri))
		{
			RescIR_Free(IR_FILE, (CHAR *)iri);
		}
	}
}

/****************************************************************************
 * FUNCTION:  OpTm_Initialize
 *
 * PURPOSE:
 *	This function is used to initialize configuration parameters
 *	and internal user information tables.
 *
 * INPUT:
 *
 * OUTPUT:
 *	IR_STATUS
 *
 * NOTES:
 *
 ***************************************************************************/

IR_STATUS	OpTm_Initialize()
{
	FILE		*fp;
	CHAR		buf[IR_MAX_LINE_SIZE];	
	SHORT		stat;

	fp = fopen(IR_CONFIG_FILE, "r");
	if (fp == NULL)
	{
		OpCf_UpdateErrorReport(IR_ERROR_FATAL, IRO_FILE_OPEN_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}
	
	/* Get implemenation id */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(implementationId));
	RESC_ASN_INIT_VS(&(implementationId));
	RescASN_AddVS(IR_FILE, &(implementationId), buf, (strlen(buf) - 1));

	/* Get implemenation name */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(implementationName));
	RESC_ASN_INIT_VS(&(implementationName));
	RescASN_AddVS(IR_FILE, &(implementationName), buf, (strlen(buf) - 1));

	/* Get implemenation version */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(implementationVersion));
	RESC_ASN_INIT_VS(&(implementationVersion));
	RescASN_AddVS(IR_FILE, &(implementationVersion), buf, (strlen(buf) - 1));

	/* Get preferred message size */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	preferredMessageSize = (LONG)atol(buf);

	/* Get maximum record size */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	maximumRecordSize = (LONG)atol(buf);

	/* Get options */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(options));
	RESC_ASN_INIT_BS(&(options));
	OpTm_HexStrToByteStr(buf);
	RescASN_AddBS(IR_FILE, &(options), buf, 7);

	/* Get protocol version */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(protocolVersion));
	RESC_ASN_INIT_BS(&(protocolVersion));
	OpTm_HexStrToByteStr(buf);
	RescASN_AddBS(IR_FILE, &(protocolVersion), buf, 2);

	/* Get user info file name */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(userInfoFileName));
	RESC_ASN_INIT_VS(&(userInfoFileName));
	RescASN_AddVS(IR_FILE, &(userInfoFileName), buf, (strlen(buf)-1));

	/* Get path of working directory */
	fgets(buf, IR_MAX_LINE_SIZE, fp);
	RESC_ASN_NO_MALLOC(&(workPath));
	RESC_ASN_INIT_VS(&(workPath));
	RescASN_AddVS(IR_FILE, &(workPath), buf, (strlen(buf) - 1));

	stat = fclose(fp);
	if (stat != 0)
	{
		OpCf_UpdateErrorReport(IR_ERROR_WARNING, IRO_FILE_CLOSE_ERROR,
		    IR_FILE);
		return (IR_FAIL);
	}

	return (IR_SUCCESS);
}
 
/****************************************************************************
 * FUNCTION:  OpTm_WorkPath
 *
 * PURPOSE:
 *	This function returns the working directory 
 *
 * INPUT:
 *
 * OUTPUT:
 *	VISIBLE_STRING		working directory	
 *
 * NOTES:
 *
 ***************************************************************************/

VISIBLE_STRING	OpTm_WorkPath()
{
	return (workPath);
}	

/****************************************************************************
 * FUNCTION:  OpTm_UserInfoFileName
 *
 * PURPOSE:
 *	This function returns the user info file name. 
 *
 * INPUT:
 *
 * OUTPUT:
 *	VISIBLE_STRING		user info file name	
 *
 * NOTES:
 *
 ***************************************************************************/

VISIBLE_STRING	OpTm_UserInfoFileName()
{
	return (userInfoFileName);
}	

/****************************************************************************
 * FUNCTION: OpTm_InitInfoFree 
 *
 * PURPOSE:
 *	This function is used to free initialization information.
 *
 * INPUT:
 *	OP_INIT_REQ_INFO	pointer to info structure
 *
 * OUTPUT:
 *
 * NOTES:
 *	There are no notes.
 *
 ***************************************************************************/

void	OpTm_InitInfoFree(OP_INIT_REQ_INFO *info)
{
	if (info != NULL)
	{
		RescASN_DestroyBS(IR_FILE, &(info->protocolVersion));
		RescASN_DestroyBS(IR_FILE, &(info->options));
		RescASN_DestroyVS(IR_FILE, &(info->implementationId));
		RescASN_DestroyVS(IR_FILE, &(info->implementationName));
		RescASN_DestroyVS(IR_FILE, &(info->implementationVersion));
		if (info->isMalloced)
		{
			RescIR_Free(IR_FILE, (CHAR *)info);
		}	
	}
}
