
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/*
 * $Header: /mnt/users/irorigin/sdl/src/resc/asn/RCS/asn_tool.c,v 1.1 1993/03/17 21:15:00 irorigin Allocated irorigin $
 * $Log: asn_tool.c,v $
 * Revision 1.1  1993/03/17  21:15:00  irorigin
 * Initial revision
 *
 */
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <ir.h>
#include <asn_incl.h>



/****************************************************************************
 * FUNCTION: ASN_OctsAdd
 *
 * PURPOSE:
 *	This function will create an octet string in SNACC format for
 *	an input character string.
 *
 * INPUT:
 *	CHAR		*buf
 *	ULONG		count
 *
 * OUTPUT:
 *	AsnOcts		*octs
 *
 * NOTES:
 *	The AsnOcts type must be exist. The AsnOcts->octs element is
 *	created by this function.
 *
 ***************************************************************************/

SHORT ASN_OctsAdd(AsnOcts *octs, char *buf, ULONG count)
/*
 * Return 0 if created.
 * Return -1 if error.
 * Return -2 if already created.
 */
{

	if (ASNOCTS_PRESENT(octs))
	{
		return (-1);
	}

	if ((octs->octs = (char *)Asn1Alloc(count)) == (char *)NULL)
	{
		return (-2);
	}

	memcpy((char *)octs->octs, (char *)buf, (int)count);
	octs->octetLen = count;
	return (0);
}


/****************************************************************************
 * FUNCTION: ASN_OctsCopy
 *
 * PURPOSE:
 *	This function will copy an AsnOcts structure to a NEW AsnOcts
 *	structure. The srcOcts item must contain the octet to copy and
 *	the destOcts must creatable from the ASN_OctsAdd function.
 *
 * INPUT:
 *	AsnOcts		*destOcts
 *	AsnOcts		*srcOcts
 *
 * OUTPUT:
 *	SHORT		0 - copied. < 0 - error.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

SHORT ASN_OctsCopy(AsnOcts *destOcts, AsnOcts *srcOcts)
/*
 * Return 0 if copied.
 * Return -1 if error.
 * Return -2 if something already exists.
 * Return -3 if no src octs.
 */
{

	if (ASNOCTS_PRESENT(srcOcts))
	{
		return (ASN_OctsAdd(destOcts, srcOcts->octs, 
		    srcOcts->octetLen));
	}
	else
	{
		return (-3);
	}
}


/****************************************************************************
 * FUNCTION: ASN_OctsDelete
 *
 * PURPOSE:
 *	This function will delete an octet string from a SNACC AsnOcts type.
 *	The string will be freed but not the AsnOcts type.
 *
 * INPUT:
 *	AsnOcts		*delOcts
 *
 * OUTPUT:
 *	SHORT		0 - deleted. < 0 - error.
 *
 * NOTES:
 *
 ***************************************************************************/

SHORT ASN_OctsDelete(AsnOcts *delOcts)
/*
 * Return 0 if deleted.
 * Return -1 if error.
 */
{

	if (ASNOCTS_PRESENT(delOcts))
	{
		FreeAsnOcts(delOcts);
		delOcts->octetLen = 0;
		return (0);
	}
	else
	{
		/* Nothing there to delete. */
		return (0);
	}
}


/****************************************************************************
 * FUNCTION: ASN_OctsGet
 *
 * PURPOSE:
 *	This function will extract an octet string into a character
 *	buffer. The character buffer will be set to point to the 
 *	start of the octet string in the AsnOcts. 
 *
 * INPUT:
 *	AsnOcts		*octs
 *
 * OUTPUT:
 *	CHAR		**buf	- pointer to octet string
 *	ULONG		count	- length of octet string
 *
 * NOTES:
 *	Any modification of the buffer data will be reflected in the 
 *	AsnOcts element as well.
 *
 ***************************************************************************/

SHORT ASN_OctsGet(char **buf, ULONG *count, AsnOcts *octs)
/*
 * Return 0 if octet string retrieved successfully.
 * Return -1 if octet string does not exits.
 * Return -2 if error obtaining octet string.
 */
{

	if (!ASNOCTS_PRESENT(octs))
	{
		*buf = (char *)NULL;
		*count = (long)0;
		return (-1);
	}

	*buf = octs->octs;
	*count = octs->octetLen;
	return(0);

}


/****************************************************************************
 * FUNCTION: ASN_OctsCompare
 *
 * PURPOSE:
 *	This function is used to test for equality only. It will compare
 *	two octet strings in AsnOcts format.
 *
 * INPUT:
 *	AsnOcts		*oct1
 *	AsnOcts		*oct2
 *
 * OUTPUT:
 *	BOOLEAN		TRUE - both octets are the same.
 *
 * NOTES:
 *
 ***************************************************************************/

BOOLEAN	ASN_OctsCompare(AsnOcts *oct1, AsnOcts *oct2)
{
	ULONG	i;
	ULONG	maxLen;
	BOOLEAN	retStatus=FALSE;

	/* Check for same length first */
	if (oct1->octetLen != oct2->octetLen)
	{
		return(retStatus);
	}
	else
	{
		/* Compare each element */
		maxLen = oct1->octetLen;
		i = 0;
		while (i < maxLen)
		{
			if (oct1->octs[i] != oct2->octs[i])
			{
				break;
			}
			i++;
		}
		if (i == maxLen)
		{
			retStatus = TRUE;
		}
		return(retStatus);
	}
}


SHORT ASN_BitsAdd(AsnBits *bits, char *buf, USHORT count)
/*
 * Return 0 if created.
 * Return -1 if error.
 * Return -2 if already created.
 */
{
	USHORT numChars;

	if (ASNBITS_PRESENT(bits))
	{
		return (-1);
	}
	
	numChars = count / 8;

	if (count % 8)
	{
		numChars++;
	}

	if ((bits->bits = (char *)Asn1Alloc(numChars)) == (char *)NULL)
	{
		return (-2);
	}

	memcpy((char *)bits->bits, buf, numChars);
	bits->bitLen = count;
	return (0);
}

SHORT ASN_BitsCopy(AsnBits *destBits, AsnBits *srcBits)
/*
 * Return 0 if copied.
 * Return -1 if error.
 * Return -2 if something already exists.
 * Return -3 if no src octs.
 */
{

	if (ASNBITS_PRESENT(srcBits))
	{
		return (ASN_BitsAdd(destBits, srcBits->bits, 
		    srcBits->bitLen));
	}
	else
	{
		return (-3);
	}
}

SHORT ASN_BitsDelete(AsnBits *delBits)
/*
 * Return 0 if deleted.
 * Return -1 if error.
 */
{
	if (ASNBITS_PRESENT(delBits))
	{
		FreeAsnBits(delBits);
		delBits->bitLen = 0;
		return(0);
	}
	else
	{
		/* Nothing there to delete. */
		return (0);
	}
}

SHORT ASN_BitsGet(char **buf, USHORT *byteCnt, USHORT *bitCnt, AsnBits *bits)
/*
 * Byte count is the total number of bytes in the string.
 * Bit count is the total number of bits in the last byte.
 * Return 0 if octet string retrieved successfully.
 * Return -1 if octet string does not exits.
 * Return -2 if error obtaining octet string.
 */
{

	if (!ASNBITS_PRESENT(bits))
	{
		*buf = (char *)NULL;
		*byteCnt = 0;
		*bitCnt = 0;
		return (-1);
	}

	*buf = bits->bits;
	*byteCnt = bits->bitLen / 8;
	*bitCnt = bits->bitLen % 8;
	return(0);

}

/****************************************************************************
 * FUNCTION: ASN_OctsCreate
 *
 * PURPOSE:
 *	This function will create an octet string in SNACC format for
 *	an input character string. This will be an uninitialized 
 *	AsnOcts type.
 *
 * INPUT:
 *	None
 *
 * OUTPUT:
 *	AsnOcts		*octs
 *
 * NOTES:
 *
 ***************************************************************************/

AsnOcts *ASN_OctsCreate()
{
	AsnOcts *octs;

	octs = (AsnOcts *)RescIR_Zmalloc(IR_FILE, sizeof(AsnOcts));

	if (octs == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "Error mallocing AsnOcts type\n");
	}
	return(octs);
}
