
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Resc ASN Bib1 utitilities
 *
 * DESCRIPTION:
 *	This file contains functions used to create and destroy basic
 *	IR BIB 1 structures.
 *
 * CONTENTS:
 *	RescUT_CloneBib1Attr - clone a bib1 attribute.
 *
 * NOTES:
 *
 *
 *
/***************************************************************************/
#include <time.h>
#include <sys/timeb.h>
#include <ir.h>
#include <resclist.h>
#include <resc.h>
#include <string.h>
#include <stdlib.h>
#include <memory.h>
#include <asn_tool.h>
#include <asn_incl.h>

void *RescUT_CloneBib1Attr(CHAR *srcFile, IR_ATTR_ELEM *srcAttr)
{
	IR_ATTR_ELEM	*destAttr;

	destAttr = (IR_ATTR_ELEM *)RescIR_Malloc(srcFile, 
	    sizeof(IR_ATTR_ELEM));

	if (destAttr == NULL)
	{
		return(destAttr);
	}

	RESC_ASN_MALLOCED(destAttr);
	destAttr->attrType = srcAttr->attrType;
	destAttr->attrValue = srcAttr->attrValue;
	return(destAttr);
}

void RescUT_PrtStr(FILE *file, CHAR *str, SHORT len)
{
	SHORT j;

	for (j = 0; j < len; j++)
	{
		putc(str[j], file);
	}
}

void RescUT_DebugVS(CHAR *src, VISIBLE_STRING *vs)
{
	/* can be vs, oi, os, or bs */
	static CHAR tmpBuf[500];
	SHORT	len;

	if (vs->len > 500)
	{
		len = 499;
	}
	else
	{
		len = vs->len;
	}

	if (vs->len == 0)
	{
		CS_LOG_PRINTS(src, "VS Debug Trying to print null pointer\n");
		return;
	}

	strncpy(tmpBuf, vs->sPtr, len);
	tmpBuf[len] = (CHAR)NULL;
	CS_LOG_PRINTV(src, "VS Debug: %s\n", tmpBuf);
}

void RescUT_ShortToVS(CHAR *src, VISIBLE_STRING *vs, SHORT val)
{
	CHAR		tmpBuf[10];

	if (vs == NULL)
	{
		CS_LOG_PRINTS(src, "Cannot convert into null VS\n");
		return;
	}
	if (vs->sPtr)
	{
		/* Destroy previous entry is need be */
		RescASN_DestroyVS(src, vs);
	}

	sprintf(tmpBuf, "%d", val);
	RescASN_AddVS(src, vs, tmpBuf, LENPLUSNULL(tmpBuf));

}

SHORT RescUT_VSToShort(CHAR *src, VISIBLE_STRING *vs)
{
	SHORT	val;

	if (!vs)
	{
		return((SHORT)-1);
	}

	if (vs->sPtr[vs->len-1] != (CHAR)NULL)
	{
		CS_LOG_PRINTS(src, "Not a null terminated string for conversion to short\n");
		return((SHORT)-1);
	}
	else
	{
		val = atoi(vs->sPtr);
		return(val);
	}
}
