/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: IR Malloc
 *
 * DESCRIPTION:
 *	This file contains the memory allocation functions used by the
 *	all IR source code.
 *
 * CONTENTS:
 *	RescIR_Malloc()		- Allocate memory
 *	RescIR_Zmalloc()	- Allocate memory preset to 0
 *	RescIR_Realloc()	- Reallocate memory.
 *	RescIR_Free()		- Free memory
 *	RescIR_StrDup()		- Copy a string.
 *
 * NOTES:
 *
 * $Header: /mnt/users/irorigin/sdl/src/resc/ir/RCS/irmalloc.c,v 1.1 1993/03/17 21:15:40 irorigin Allocated irorigin $
 * $Log: irmalloc.c,v $
 * Revision 1.1  1993/03/17  21:15:40  irorigin
 * Initial revision
 *
 *
/***************************************************************************/
#ifdef MEM_STDLIB
#undef MEM_STDLIB
#endif

#ifdef MEM_XVTLIB
#undef MEM_XVTLIB
#endif

#define MEM_XVTLIB

#include <ir.h>
#include <resc.h>
#include <ir_cs.h>

#ifdef MEM_XVTLIB
#include <xvt.h>
#endif
#include <malloc.h>
#include <string.h>
#include <process.h>

/****************************************************************************
 * FUNCTION: RescIR_Malloc
 *
 * PURPOSE:
 *
 *	All memory allocation performed by any IR function must use
 *	this function.
 *
 * INPUT:
 *	CHAR		*src;	use IR_FILE macro to determine src location
 *	LONG 		size; 	size in bytes.
 *
 * OUTPUT:
 *	This function returns a character pointer to the data block
 *	that was allocated. A Null pointer will be returned if the
 *	data block could not be malloced.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

CHAR *RescIR_Malloc(CHAR *src, ULONG size)
{

	void *ptr;

#ifdef MEM_XVTLIB
	ptr = xvt_zmalloc((size_t)size);
#endif

#ifdef MEM_STDLIB
	/* Use calloc for now to ensure clean memory */
	ptr = (CHAR *)calloc((unsigned int) size, 1);
#endif

	return(ptr);
}

/****************************************************************************
 * FUNCTION: RescIR_Zmalloc
 *
 * PURPOSE:
 *
 *	All memory allocation performed by any IR function must use
 *	this function.
 *
 * INPUT:
 *	CHAR		*src;	use IR_FILE macro to determine src location
 *	LONG 		size; 	size in bytes.
 *
 * OUTPUT:
 *	This function returns a character pointer to the data block
 *	that was allocated. A Null pointer will be returned if the
 *	data block could not be malloced.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

CHAR *RescIR_Zmalloc(CHAR *src, ULONG size)
{
	void *ptr;

#ifdef MEM_XVTLIB
	ptr = xvt_zmalloc((size_t)size);
#endif
#ifdef MEM_STDLIB
	ptr = (CHAR *)calloc((unsigned int)size, 1);
#endif

	return(ptr);
}

/****************************************************************************
 * FUNCTION: RescIR_Realloc
 *
 * PURPOSE:
 *
 *	All memory re-allocation performed by any IR function must use
 *	this function.
 *
 * INPUT:
 *	CHAR		*src;	use IR_FILE macro to determine src location
 *	CHAR		*curBlock; 	pointer to current block
 *	LONG 		size; 		size in bytes.
 *
 * OUTPUT:
 *	This function returns a character pointer to the data block
 *	that was allocated. A Null pointer will be returned if the
 *	data block could not be malloced.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

CHAR *RescIR_Realloc(CHAR *src, CHAR *curBlock, ULONG size)
{

	void *ptr;

#ifdef MEM_XVTLIB
	ptr = xvt_realloc((void *)curBlock, (size_t)size);
#endif

#ifdef MEM_STDLIB
	ptr = realloc((char *)curBlock, (unsigned int)size);
#endif

	return(ptr);
}

/****************************************************************************
 * FUNCTION: RescIR_Free
 *
 * PURPOSE:
 *
 *	All memory that is to be freed by any IR function must use
 *	this function.
 *
 * INPUT:
 *	CHAR		*src;	use IR_FILE macro to determine src location
 *	CHAR		*curBlock; 	pointer to current block
 *
 * OUTPUT:
 *	This function has no return value.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

CHAR *RescIR_Free(CHAR *src, CHAR *curBlock)
{
	char buf[IR_LEN_100];

	strcpy(buf, src);

	if (curBlock != NULL)
	{
	
#ifdef MEM_XVTLIB
		xvt_free((void *)curBlock);
#endif
#ifdef MEM_STDLIB
		free((char *)curBlk);
#endif
	}
	else
	{
		CS_LOG_PRINTV(IR_FILE, "Free error at: %s\n", buf);
	}
	
	return((char *)NULL);
}

/*****************************************************************************
 * FUNCTION RescIR_ExpandFile
 *
 * PURPOSE:
 *	The RescIR_ExpandFile function is used in conjunction with the IR_FILE
 *	macro.  It it used to combine the filename and the current line
 *	number to form one string.
 *
 * NOTES:
 *
 *****************************************************************************/

CHAR	*RescIR_ExpandFile(int lineNum, char *fileName)
{
	static char FileName[IR_LEN_100];

	sprintf(FileName, "%s(%d)",fileName, lineNum);
	return(FileName);
}

/****************************************************************************
 * FUNCTION: RescIR_StrDup
 *
 * PURPOSE:
 *
 *	To duplicate a string.
 *
 * INPUT:
 *	CHAR		*str;	string
 *
 * OUTPUT:
 *	This function returns a character pointer to the data block
 *	that was allocated and now contains a copy of the input string.
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

CHAR *RescIR_StrDup(CHAR *src, CHAR *str)
{
	CHAR *ptr;

	ptr = (CHAR *)RescIR_Malloc(src, strlen(str) + 1);
	strcpy(ptr, str);
	return (ptr);
}
