
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Abort Association
 *
 * DESCRIPTION:
 *
 *	The Abort Association CSU will get association input from the 
 *	user in order to abort an association.
 *
 * CONTENTS:
 *	UaAc_AbortAssociation
 *
 * NOTES:
 *	This function should only be called from within the User
 *	application layer of the IR Kernel-Origin.
 *
 * $Header: /mnt/users/irorigin/sdl/src/ua/ac/RCS/abort.c,v 1.1 1993/03/17 21:40:58 irorigin Allocated irorigin $
 * $Log: abort.c,v $
 * Revision 1.1  1993/03/17  21:40:58  irorigin
 * Initial revision
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <ua/dialog.h>

/****************************************************************************
 * FUNCTION: UaAc_AbortAssociation
 *
 * PURPOSE:
 *
 * 	The User Application Association Control Abort Association
 *	obtains association input from the user and then attempts to 
 *	use that input to abort that association.
 *
 * INPUT:
 *	This function requires the association Id (IR_ASSOC_ID) which
 *	is to be aborted.
 *
 * OUTPUT:
 *	This function returns an IR_STATUS with returns a non-zero
 *	value on error. A call to UaHec_ErrorControl is required
 *	to inform the user of the problem.
 *
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

IR_STATUS UaAc_AbortAssociation(IR_ASSOC_ID assocId)
{
	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;	/* Function Return
							 * Code */
	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);

	/*
 	 * Initialize the dialog information.
	 */
	returnCode = IR_SUCCESS;
	/*
	 * Use the user Info to set up call to IRO_Assocoiation.
 	 */
	assocReq.nameType = IRO_ASSOC_ADDR_TARGET_ID;
	assocReq.assocId = assocId;

	/*
	 * Attempt to Abort the Session.
	 */
	returnCode = IRO_Association(UaEp_InfoUserId(), IRO_ASSOC_ABORT,
	    &assocReq, 
	    &assocReply);
	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE,
		    IRO_ASSOCIATION_FUNC, IRO_ASSOC_ABORT);
	}
	UaAc_CommStatusUpdate();
	UaAc_UpdateStatus();
	return(returnCode);
}
