/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Association Control GUI List functions.
 *
 * DESCRIPTION:
 *
 *	This unit is used to generate and manipulate SLIST info 
 *	for the gui dialog boxes that require association information.
 *
 * CONTENTS:
 *	UaAc_CMHandleList
 *	UaAc_CMCreateList
 *
 * NOTES:
 *	This function should only be called from within the User
 *	application layer of the IR Kernel-Origin.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <ua/dialog.h>

void UaAc_CMHandleList(WINDOW listWin, UAEP_DIALOG *info, SLIST list)
{
	long	data;

	if (EpDialog_GetSelListData(listWin, list, &data) != IR_SUCCESS)
	{
		info->data.makeConnect.targetId = IR_UNKNOWN_ID_TYPE;
	}
	else
	{
		info->data.makeConnect.targetId = (IR_TARGET_ID)data;
	}
}

void UaAc_CMCreateList(SLIST connectList, SLIST targetList)
{
	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;
	SHORT				count;
	IRO_ASSOC_STRUCT		*assocStruct;
	IR_USER_TARGET_INFO		*targetInfo;
	CHAR				tmpBuf[100];
	IRO_USER_INFO_REQ		infoReq;
	IRO_USER_INFO_REPLY		infoReply;
	CHAR				buf[20];


	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);
	RESC_ASN_NO_MALLOC(&infoReq);
	RESC_ASN_NO_MALLOC(&infoReply);

	if ((!connectList) || (!targetList))
	{
		xvt_note("Cannot work with uninitialized lists");
		return;
	}

	/*
	 * Get the connected associations from the IRO api.
	 */
	returnCode = IRO_Association(UaEp_InfoUserId(),
	    IRO_ASSOC_GET_ASSOC_INFO,
	    &assocReq, 
	    &assocReply);

	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE, IRO_ASSOCIATION_FUNC, 
		    IRO_ASSOC_GET_ASSOC_INFO);
	}
	else if (RESC_LIST_SIZE(assocReply.activeAssociations) > 0)
	{
		RESC_LIST_SET_FIRST(assocReply.activeAssociations);
		for (count = 0;
		    count < RESC_LIST_SIZE(assocReply.activeAssociations);
		    count++)
		{

			assocStruct = 
			    RESC_LIST_GET(assocReply.activeAssociations);
			
			strncpy(tmpBuf, assocStruct->name.sPtr,
			    assocStruct->name.len);
			tmpBuf[assocStruct->name.len] = (CHAR)NULL;
			sprintf(buf,"%d", assocStruct->assocId);
			strcat(tmpBuf, buf);
			slist_add_sorted(connectList, tmpBuf,
			    assocStruct->assocId,
			    FALSE, FALSE);
			RESC_LIST_SET_NEXT(assocReply.activeAssociations);
		}
	}

	/* 
 	 * Insert the targets known to the user into the target list.
	 */
	infoReq.myUserId = UaEp_InfoUserId();
	
	returnCode = IRO_UserInfo(IRO_UI_TARGETS, &infoReq, &infoReply);

	if (returnCode != IR_SUCCESS) 
	{
		xvt_error("cannot access target info for known user");
		return;
	}
	else if (RESC_LIST_SIZE(infoReply.targetInfo) > 0)
	{
		/* Add known targets to target list. */
		RESC_LIST_SET_FIRST(infoReply.targetInfo);
		for (count = 0;
		    count < RESC_LIST_SIZE(infoReply.targetInfo);
		    count++)
		{
			targetInfo = RESC_LIST_GET(infoReply.targetInfo);
			strncpy(tmpBuf, targetInfo->targetName.sPtr,
			    targetInfo->targetName.len);
			tmpBuf[targetInfo->targetName.len] = (CHAR)NULL;
			slist_add_sorted(targetList, tmpBuf,
			    targetInfo->targetId,
			    FALSE, FALSE);
			RESC_LIST_SET_NEXT(infoReply.targetInfo);
		}
	}
	
}
