
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Break Association
 *
 * DESCRIPTION:
 *
 *	The Break Association CSU will get association input from the 
 *	user in order to break an assocation.
 *
 * CONTENTS:
 *	UaAc_BreakAssociation
 *
 * NOTES:
 *	This function should only be called from within the User
 *	application layer of the IR Kernel-Origin.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <ua/dialog.h>

/****************************************************************************
 * FUNCTION: UaAc_BreakAssociation
 *
 * PURPOSE:
 *
 * 	The User Application Association Control Break Association
 *	obtains association input from the user and then attempts to 
 *	use that input to terminate that association with the forign 
 *	target specified.
 *
 * INPUT:
 *	A valid preset UAEP_DIALOG pointer.
 *
 * OUTPUT:
 * 	This function has no return value.
 *
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

#ifdef AMICUS
void UaAc_BreakTargetConnection(IR_TARGET_ID targetId)
{
	UAEP_DIALOG dlgRqst;

	dlgRqst.data.breakConnect.targetId = targetId;

	UaAc_BreakAssociation(&dlgRqst);
}
#endif
void UaAc_BreakAssociation(UAEP_DIALOG *dlgRqst)
{
	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;	/* Function Return
							 * Code */
	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);


	if (dlgRqst->data.breakConnect.abort == TRUE)
	{
		UaAc_AbortAssociation(dlgRqst->data.breakConnect.assocId);
		return;
	}

	/*
	 * Use the user Info to set up call to IRO_Assocoiation.
 	 */
	assocReq.assocId = dlgRqst->data.breakConnect.assocId;

	/*
	 * Attempt to Break the Session.
	 */
	returnCode = IRO_Association(UaEp_InfoUserId(), IRO_ASSOC_TERMINATE,
	    &assocReq, 
	    &assocReply);
	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE,
		    IRO_ASSOCIATION_FUNC, IRO_ASSOC_TERMINATE);
	}

	UaAc_CommStatusUpdate();
	UaAc_UpdateStatus();
	return;
}
