/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT: Make Association
 *
 * DESCRIPTION:
 *
 *	The Make Association CSU will get association input from the 
 *	user in order to establish an assocation.
 *
 * CONTENTS:
 *	UaAc_MakeAssociation
 *
 * NOTES:
 *	This function should only be called from within the User
 *	application layer of the IR Kernel-Origin.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>

/****************************************************************************
 * FUNCTION: UaAc_MakeAssociation
 *
 * PURPOSE:
 *
 * 	The User Application Association Control Make Association
 *	obtains association input from the user and then attempts to 
 *	use that input to establish that association with the forign 
 *	target specified.
 *
 * INPUT:
 *	This function requires a preset UAEP_DIALOG pointer as input.
 *
 * OUTPUT:
 *	No return value.
 *
 *
 * NOTES:
 *	Nothing of Note.
 *
 ***************************************************************************/

#ifdef AMICUS
void UaAc_ConnectToTarget(IR_TARGET_ID targetId)
{
	UAEP_DIALOG dlgRqst;

	dlgRqst.data.makeConnect.targetId = targetId;

	UaAc_MakeAssociation(&dlgRqst);
}
#endif

void UaAc_MakeAssociation(UAEP_DIALOG *dlgRqst)
{
	IRO_ASSOC_REQ 			assocReq;	/* IRO Req. Info */
	IRO_ASSOC_REPLY			assocReply;	/* IRO Reply not used */
	IR_STATUS			returnCode;	/* Function Return
							 * Code */
	IR_LIST				assocList;	/* list info */
	IRO_ASSOC_STRUCT		*assocEntry;	/* list entry */

	if (dlgRqst->data.makeConnect.targetId == IR_UNKNOWN_ID_TYPE)
	{

#ifdef XVT_MSDOS
		xvt_note("Unknown target selected. Try again.");
#else
		CS_LOG_PRINTS(IR_FILE, "Unknown target selected. Try again.");
#endif
		return;
	}

	/* Initialize the local variable assocList */
	RESC_ASN_NO_MALLOC(&assocList);
	RESC_LIST_INIT(&assocList);

	/*
	 * Use the user Info to set up call to IRO_Assocoiation.
	 */
	RESC_ASN_NO_MALLOC(&assocReq);
	RESC_ASN_NO_MALLOC(&assocReply);
#ifdef XVT_MSDOS
	assocReq.assocId = UaCs_GetNewAssocId();
#else
	assocReq.assocId = IrAp_GetNewAssocId();
#endif
	assocReq.nameType = IRO_ASSOC_ADDR_TARGET_ID;
	assocReq.targetId = dlgRqst->data.makeConnect.targetId;

	/*
	 * Attempt to establish the Session.
	 */
	returnCode = IRO_Association(UaEp_InfoUserId(), IRO_ASSOC_ESTABLISH,
	    &assocReq, 
	    &assocReply);

	UaAc_CommStatusUpdate();
	if (returnCode != IR_SUCCESS)
	{
		UaHec_ReportIROError(IR_FILE, IRO_ASSOCIATION_FUNC, 
		    IRO_ASSOC_ESTABLISH);
#ifdef XVT_MSDOS
		UaCs_DeleteAssocId(assocReq.assocId);
#else
		IrAp_DeleteAssocId(assocReq.assocId);
#endif
	}
	else
	{
#ifdef XVT_MSDOS
		UaAc_UpdateStatus();
		if (UaAc_GetAssociationInfo(UAAC_GET_ONE_ASSOC,
		    assocReq.assocId, &assocList) == IR_FAIL)
		{
			CS_LOG_PRINTV(IR_FILE, "Couldn't get info for assoc %d\n",
			    assocReq.assocId);
			return;
		}
		else
		{
			if (RESC_LIST_SIZE(&assocList) > 0)
			{
				RESC_LIST_SET_FIRST(&assocList);
				assocEntry = RESC_LIST_GET(&assocList);
				UaHec_AddAssocName(assocReq.assocId,
			    	    &assocEntry->hostName);
				UaAc_ResetTimer(assocReq.assocId);
			}
			else
			{
				CS_LOG_PRINTV(IR_FILE, 
				    "No association found for assoc Id = %d\n",
				    assocReq.assocId);
			}

			RescList_Destroy(IR_FILE, &assocList);
		}
#endif
	}
	return;
}
