/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <resclist.h>
#include <uaep.h>

UaArc_HandleList(WINDOW listWin, UAEP_DIALOG *info, SLIST targetList)
{
	WINDOW ctlWin;
	WINDOW targetWin;
	long  data;
	IRO_ASSOC_FULL_INFO_STRUCT infoStruct;
	BOOLEAN doRsrcRpt;
	BOOLEAN doRsrcCtl;
	BOOLEAN doAbort;
	BOOLEAN doStop;


	if (EpDialog_GetSelListData(listWin, targetList, &data) != IR_SUCCESS)
	{
		info->data.arcStatus.assocId = IR_UNKNOWN_ID_TYPE;
		ctlWin = get_ctl_window(targetWin, CSRS_ASSOC_INFO);
	 	enable_window(ctlWin, FALSE);
		return(0);
	}
	else
	{
		info->data.arcStatus.assocId = (IR_ASSOC_ID)data;
	}

	/* Get the related information for the select association */
	RESC_ASN_NO_MALLOC(&infoStruct);

	if (UaAc_GetAssociationInfoFull(info->data.arcStatus.assocId,
	    &infoStruct) == IR_FAIL)
	{
		info->data.arcStatus.assocId = IR_UNKNOWN_ID_TYPE;
		return(0);
	}

	/* 
	 * Use the info struct to determine which control buttons
 	 * are valid for this association.
	 */

	doRsrcRpt = FALSE;
	doRsrcCtl = FALSE;
	doStop = FALSE;
	doAbort = TRUE;

	switch(infoStruct.assocStruct.state)
	{
	case IRO_ASSOC_STATE_SEARCH:
	case IRO_ASSOC_STATE_PRESENT:
	case IRO_ASSOC_STATE_DELETE:
		if (infoStruct.initResp.triggerResourceCtrl == TRUE)
		{
			doRsrcCtl = TRUE;
			doRsrcRpt = TRUE;
			doStop = TRUE;
		}
		break;

	case IRO_ASSOC_STATE_INIT:
	case IRO_ASSOC_STATE_CONNECT:
		break;

	case IRO_ASSOC_STATE_RSRC_RPRT:
		if (infoStruct.initResp.triggerResourceCtrl == TRUE)
		{
			doStop = TRUE;
		}
		break;

	case IRO_ASSOC_STATE_IDLE:
		if (infoStruct.initResp.resourceReport == TRUE)
		{
			doRsrcRpt = TRUE;
		}
		doAbort = FALSE;
		break;
	default:
		break;
	}

	targetWin = get_parent(listWin);
	ctlWin = get_ctl_window(targetWin, CSRS_RESOURCE_RPT_BUTTON);
	enable_window(ctlWin, doRsrcRpt);

	ctlWin = get_ctl_window(targetWin, CSRS_RESOURCE_CNTL);
	enable_window(ctlWin, doRsrcCtl);

	ctlWin = get_ctl_window(targetWin, CSRS_ABORT_ASSOC);
	enable_window(ctlWin, doAbort);

	ctlWin = get_ctl_window(targetWin, CSRS_STOP_OPERATION);
	enable_window(ctlWin, doStop);

	/* Info is always valid if there is a least one entry in the list */
	ctlWin = get_ctl_window(targetWin, CSRS_ASSOC_INFO);
	enable_window(ctlWin, TRUE);

	UaAc_DestroyFullInfoStruct(&infoStruct);

	return(1);
}


IR_STATUS UaArc_CreateList(SLIST targetList, UAEP_DIALOG *info)
{
	SHORT			index;
	IR_LIST			assocList;	/* List of IRO_ASSOC_STRUCT */
	EP_DLG_EDIT_TYPE	targetString;
	IRO_ASSOC_STRUCT	*tmpAssoc;
	CHAR			tmpBuf[100];

	RESC_ASN_NO_MALLOC(&assocList);
	RESC_LIST_INIT(&assocList);

	/*
	 * Get the active association list.
	 */
	UaAc_GetAssociationInfo(UAAC_GET_ALL_ASSOCS, IR_UNKNOWN_ID_TYPE,
	    &assocList);

	/*
	 * Allocate space for assocation list to sort.
	 */
	if (RESC_LIST_SIZE(&assocList) == 0)
	{
		/* No entries to display */
		CS_LOG_PRINTS(IR_FILE, "Tried to create status list but no associations exist.\n");
		return (IR_FAIL);
	}

	info->data.arcStatus.assocId = IR_UNKNOWN_ID_TYPE;

	/*
	 * Add items to Slist.
	 */
	RESC_LIST_SET_FIRST(&assocList);
	for (index = 0; index < RESC_LIST_SIZE(&assocList); index++)
	{
		tmpAssoc = RESC_LIST_GET(&assocList);

		strncpy(tmpBuf, tmpAssoc->name.sPtr, tmpAssoc->name.len);
		tmpBuf[tmpAssoc->name.len] = (CHAR)NULL;
		sprintf(targetString, "%c %s",
		    GET_ASSOC_CHAR(tmpAssoc->state), tmpBuf);
		/*
		 * Add the target information to the target list.
		 * the index indicates where the target data is stored
		 * in the data table.
		 */
		slist_add(targetList,
		    (SLIST_ELT)NULL,
		    (char *)targetString,
		    (long)(tmpAssoc->assocId));

		RESC_LIST_SET_NEXT(&assocList);
	}

	/* Free the association list. */
	RescList_Destroy(IR_FILE, &assocList);

	return(IR_SUCCESS);

}

BOOLEAN UaArc_OkToGo(IR_ASSOC_ID assocId)
{
	if (assocId != (IR_ASSOC_ID)IR_UNKNOWN_ID_TYPE)
	{
		return (TRUE);
	}
	xvt_beep();
	return (FALSE);
}
