/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Dialog handler
 *
 * DESCRIPTION:
 *	The Event Processing Dialog Handler CSU starts the various
 *	dialogs with the appropriate function calls.
 *
 * CONTENTS:
 *	UaEp_Dialog
 *
 * NOTES:
 *	This function does not handle all operations for dialogs
 * 	it simply handles the basic issue of starting the various
 *	types of dialogs.
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ua.h>
#include "xvt.h"
#include "xvtmenu.h"
#include "uaep.h"

/****************************************************************************
 * FUNCTION:  UaEp_Dialog
 *
 * PURPOSE:
 *	The User Application Event Processing Dialog function invokes
 *	the appropriate dialog based on the input obtained.
 *
 * INPUT:
 *	This function requires a UAEP_DIALOG_TYPE which indicates the
 *	type of dialog to start. (See uaep.h and  uaepappl.h)  and
 *	a dialog structure containing the information to return or
 *	display for the user.(See ua.h and ep_dialog.h)
 *
 * OUTPUT:
 *	There is no output from this CSU.
 *
 * NOTES:
 *	It is on a dialog by dialog basis as to whether the data is
 *	used for input/output or both. It is highly recommended that
 *	all dialogs used are modless.
 ***************************************************************************/

#if 0
IR_STATUS UaEp_Dialog(UAEP_DIALOG_TYPE type, UAEP_DIALOG *data)
{

	/*
 	 * Currently all of the dialogs are invoked through
	 * the function "invoke" This function is provided as
 	 * a result of creating the dialogs with XVT-Design.
	 */
	(void)invoke((int)type, (long)data);
	return(IR_SUCCESS);
}
#endif

char *EpDialog_IsInList (SLIST list, long value)
{
	long	    compare;
	SLIST_ELT   elem;
	char	    *returnVal;

	for (elem = slist_first(list);
	    elem != (SLIST_ELT)NULL;
	    elem = slist_next(list, elem))
	{
		returnVal = slist_get(list, elem, &compare);
		if (compare == value)
		{
			return returnVal;
		}
	}
	return (char *)NULL;
}

IR_STATUS EpDialog_GetSelListData(WINDOW win, SLIST list, long *data)
{
	int index;

	index = win_list_get_sel_index(win);
	if ((index < 0) || (index > win_list_count_all(win)))
	{
		return(IR_FAIL);
	}
	slist_elt(list, index, data);
	return(IR_SUCCESS);
}

IR_STATUS EpDialog_SetSelListData(WINDOW win, SLIST list, long data)
{
	BOOLEAN notFound;
	long	compare;
	SHORT 	index;
	IR_STATUS returnCode;

	notFound = TRUE;
	returnCode = IR_FAIL;

	for (index = 0; (index < slist_count(list)) && notFound; index++)
	{
		(void)slist_elt(list, index, &compare);
		if (compare == data)
		{
			win_list_set_sel(win, index, TRUE);
			notFound = FALSE;
			returnCode = IR_SUCCESS;
		}
	}
	return(returnCode);
}

SHORT EpDialog_GetEditShort(WINDOW win)
{
	EP_DLG_EDIT_TYPE displayInt;
	int returnVal;

	returnVal = -1;
	get_title(win, displayInt, sizeof(EP_DLG_EDIT_TYPE));
	if (strlen(displayInt) > 0)
	{
		returnVal = (SHORT)atoi(displayInt);
	}
	return(returnVal);
}

#define SZ_PASSWD   20
#define SZ_NAME     20
#define SZ_DIGIT    4

void EpDialog_HidePasswd(WINDOW cwin, char *passwd)
{
	int i;
	static EP_DLG_EDIT_TYPE buf;

	get_title(cwin, buf, sizeof(EP_DLG_EDIT_TYPE));

	if ((int)strlen(buf) > SZ_PASSWD)
	{
		buf[SZ_PASSWD] = '\0';
		set_title(cwin, buf);
		xvt_beep();
	}

	if ((int)strlen(passwd) == 0 && strlen(buf) > 1)
	{
		buf[0] = '\0';
	}

	buf[SZ_PASSWD] = '\0';
	for (i = 0; buf[i] != '\0'; i++)
	{
		if (buf[i] != '*')
		{
			passwd[i] = buf[i];
			buf[i] = '*';
		}
	}
	passwd[i] = '\0';
	set_title(cwin, buf);
	win_select_item_text(cwin, i, i);
}

void EpDialog_NoWhiteSpace(WINDOW cwin, char *buf)
{
	int i;

	get_title(cwin, buf, sizeof(EP_DLG_EDIT_TYPE));
	if ((int)strlen(buf) > SZ_NAME)
	{
		buf[SZ_NAME] = '\0';
		set_title(cwin, buf);
		xvt_beep();
	}
	buf[SZ_NAME] = '\0';
	for (i = 0; buf[i] != '\0'; i++)
	{
		if (isspace(buf[i]))
		{
			xvt_beep();
			buf[i] = '\0';
			set_title(cwin, buf);
			win_select_item_text(cwin, i, i);
			break;
		}
	}
}

void EpDialog_AlphaOnly(WINDOW cwin)
{
	int i;
	EP_DLG_EDIT_TYPE buf;

	get_title(cwin, buf, sizeof(EP_DLG_EDIT_TYPE));
	if ((int)strlen(buf) > SZ_NAME)
	{
		buf[SZ_NAME] = '\0';
		set_title(cwin, buf);
		xvt_beep();
	}
	buf[SZ_NAME] = '\0';
	for (i = 0; buf[i] != '\0'; i++)
	{
		if (!isalpha(buf[i]))
		{
			xvt_beep();
			buf[i] = '\0';
			set_title(cwin, buf);
			win_select_item_text(cwin, i, i);
			break;
		}
	}
}

void EpDialog_IsDigit(WINDOW cwin)
{
	int i;
	EP_DLG_EDIT_TYPE buf;

	get_title(cwin, buf, sizeof(EP_DLG_EDIT_TYPE));
	if ((int)strlen(buf) > SZ_DIGIT)
	{
		buf[SZ_DIGIT] = '\0';
		set_title(cwin, buf);
		xvt_beep();
	}
	buf[SZ_DIGIT] = '\0';
	for (i = 0; buf[i] != '\0'; i++)
	{
		if (!isdigit(buf[i]))
		{
			xvt_beep();
			buf[i] = '\0';
			set_title(cwin, buf);
			win_select_item_text(cwin, i, i);
			break;
		}
	}
}
