/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <ir_net.h>

typedef struct 
{
	HWND	hWnd;
	USHORT  event;
	USHORT  wParam;
	ULONG	lParam;
} NET_MSG_STRUCT;

int UaEp_CommEvent (WINDOW win, EVENT *ep)
{
	IRO_COMM_EVENT_REQ   commEvent;
	IRO_COMM_EVENT_REPLY *iroReply;
	IRO_COMM_EVENT_REPLY commReply;
	NET_MSG_STRUCT		*netMsg;

	RESC_ASN_NO_MALLOC(&commEvent);
	RESC_ASN_NO_MALLOC(&commReply);

	/*
	 * Pass the information sent to the assoc window to the comm
	 * event handler to process the event.
	 */
	commEvent.assocId = (IR_ASSOC_ID)ep->v.user.id;
	netMsg = (NET_MSG_STRUCT *)ep->v.user.ptr;
	commEvent.assocId = netMsg->hWnd;
	commEvent.eventInfo = (CHAR *)ep->v.user.ptr;
	UaAc_ResetTimer(commEvent.assocId);
	if (IRO_CommEvent(&commEvent, &iroReply) != IR_SUCCESS)
	{
		xvt_note("Communications Failure for Assoc #%d",
		    commEvent.assocId);
		return(0);
	}
	/* Copy the iro reply to the local comm reply structure */
	memcpy((void *)&commReply, (void *)iroReply,
	    sizeof(IRO_COMM_EVENT_REPLY));
	CS_LOG_PRINTV(IR_FILE, "ua comm event = %d ", commReply.commReplyType);
	CS_LOG_PRINTV("", "for assocId = %d\n", commReply.assocId);
	switch (commReply.commReplyType)
	{
	case IR_COMM_DATA_ERROR:
		xvt_note("IRO_CommData returned an error\n");
		break;
	case IR_COMM_DATA_NO_DATA:
		break;

	case IR_COMM_DATA_CONNECT:
	case IR_COMM_DATA_RELEASE:
	case IR_COMM_DATA_ABORT:
	case IR_COMM_DATA_INIT_SRVC:
	case IR_COMM_DATA_INIT_FAIL:
	case IR_COMM_DATA_REL_FAIL:
	case IR_COMM_DATA_ABORT_FAIL:
	case IR_COMM_DATA_CONN_REFUSED:
	case IR_COMM_DATA_UNKNOWN_HOST:
	case IR_COMM_DATA_HOST_NOT_FOUND:
	case IR_COMM_DATA_PROTOCOL_ERR:
		UaAc_UpdateStatus();
		(void)UaAc_CommEvent(
		    (UAAC_COMM_EVENT_DATA_TYPE)commReply.commReplyType,
		    &commReply);
		UaAc_CommStatusUpdate();
		break;

	case IR_COMM_DATA_SEARCH_SRVC:
	case IR_COMM_DATA_SRCH_FAIL:
		(void)UaSc_CommEvent(
		    (UASC_COMM_EVENT_DATA_TYPE)commReply.commReplyType,
		    &commReply);
		UaAc_CommStatusUpdate();
		break;

	case IR_COMM_DATA_PRESENT_SRVC:
	case IR_COMM_DATA_PRSNT_FAIL:
		/* UaPc_CommEvent */
		(void)UaPc_CommEvent(
		    (UAPC_COMM_EVENT_DATA_TYPE)commReply.commReplyType,
		    &commReply);
		UaAc_CommStatusUpdate();
		break;
	case IR_COMM_DATA_DELETE_SRVC:
		/* UaPc_CommEvent */
		(void)UaRdc_CommEvent(
		    commReply.commReplyType,
		    &commReply);
		UaAc_CommStatusUpdate();
		break;

	case IR_COMM_DATA_ACCS_SRVC:
	case IR_COMM_DATA_RSRC_CNTL_SRVC:
	case IR_COMM_DATA_RSRC_RPT_SRVC:
		(void)UaArc_CommEvent(
		    (UAARC_COMM_EVENT_DATA_TYPE)commReply.commReplyType,
		    &commReply);
		UaAc_CommStatusUpdate();
		break;
	default:
		xvt_note("Unknown Communciation reply type %d",
		    commReply.commReplyType);
		break;
	}
	return(0);
}

