/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 ************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <ua.h>
#include "xvt.h"
#include "xvtmenu.h"
#include <resc.h>
#include "uaep.h"



/*
 * Menu Items activiation indicatiors.
 */
static BOOLEAN MFileNew = FALSE;
static BOOLEAN MFileOpen = FALSE;
static BOOLEAN MFileClose = FALSE;
static BOOLEAN MFileSave = FALSE;
static BOOLEAN MFileSaveAs = FALSE;
static BOOLEAN MFileRevert = FALSE;
static BOOLEAN MFilePgSetup = FALSE;
static BOOLEAN MFilePrint = FALSE;
static BOOLEAN MEditUndo = FALSE;
static BOOLEAN MEditCut = FALSE;
static BOOLEAN MEditCopy = FALSE;
static BOOLEAN MEditPaste = FALSE;
static BOOLEAN MEditClear = FALSE;
static BOOLEAN MEditSelAll = FALSE;
static BOOLEAN MEditClipboard = FALSE;
static BOOLEAN MConnectBreak = FALSE;
static BOOLEAN MConnectList = FALSE;
static BOOLEAN MConnectStatus = FALSE;
static BOOLEAN MSearchDatabase = FALSE;
static BOOLEAN MSearchClear = FALSE;
static BOOLEAN MSearchSend = FALSE;
static BOOLEAN MSearchHistory = FALSE;
static BOOLEAN MSearchStatus = FALSE;
static BOOLEAN MResultShow = FALSE;
static BOOLEAN MResultDisplay = FALSE;
static BOOLEAN MResultFetch = FALSE;
static BOOLEAN MResultList = FALSE;
static BOOLEAN MResultDelete = FALSE;
static BOOLEAN MResultStatus = FALSE;
static BOOLEAN MOptionsDefaults = FALSE;
static BOOLEAN MOptionsUsers = FALSE;
static BOOLEAN MOptionsMessages  = FALSE;

void UaEp_MenuSetStatus(
    MENU_TAG tag,
    BOOLEAN active)
{
	switch (tag) 
	{
	/*case M_FILE_NEW: /* Menu "New" */
		MFileNew = active;
		break;
	/*case M_FILE_OPEN: /* Menu "Open" */
		MFileOpen = active;
		break;
	/*case M_FILE_CLOSE: /* Menu "Close" */
		MFileClose = active;
		break;
	case M_FILE_SAVE: /* Menu "Save" */
		MFileSave = active;
		break;
	/*case M_FILE_SAVE_AS: /* Menu "Save As..." */
		MFileSaveAs = active;
		break;
	/*case M_FILE_REVERT: /* Menu "Revert" */
		MFileRevert = active;
		break;
	/*case M_FILE_PG_SETUP: /* Menu "Page Setup..." */
		MFilePgSetup = active;
		break;
	/*case M_FILE_PRINT: /* Menu "Print" */
		MFilePrint = active;
		break;
	case M_FILE_QUIT:
		/* This is always active. */
		break;
	case M_FILE_ABOUT_IR: /* Menu "About..." */
		/* This is always active. */
		break;
	case M_EDIT_UNDO: /* Menu "Undo" */
		MEditUndo = active;
		break;
	case M_EDIT_CUT: /* Menu "Cut" */
		MEditCut = active;
		break;
	case M_EDIT_COPY: /* Menu "Copy" */
		MEditCopy = active;
		break;
	case M_EDIT_PASTE: /* Menu "Paste" */
		MEditPaste = active;
		break;
	case M_EDIT_CLEAR: /* Menu "Clear" */
		MEditClear = active;
		break;
	case M_EDIT_SEL_ALL: /* Menu "Select All" */
		MEditSelAll = active;
		break;
	case M_EDIT_CLIPBOARD: /* Menu "Show Clipboard" */
		MEditClipboard = active;
		break;
	case M_CONNECT_MAKE: /* Menu "Make" */
		/* This is always active. */
		break;
	case M_CONNECT_BREAK: /* Menu "Break" */
		MConnectBreak = active;
		MSearchDatabase = MConnectBreak;
		MConnectStatus = MConnectBreak;
		MSearchStatus = MConnectBreak;
		MResultStatus = MConnectBreak;
		break;
	case M_CONNECT_STATUS: /* Menu "Status" */
		MConnectStatus = active;
		MSearchStatus = MConnectStatus;
		MResultStatus = MConnectStatus;
		break;
	case M_SEARCH_QUERY: /* Menu "Query" */
		/* Are Always active. */
		break;
	case M_SEARCH_CLEAR: /* Menu "Clear" */
		MSearchClear = active;
		break;
	case M_SEARCH_DATABASE: /* Menu "Database" */
		MSearchDatabase = active;
		break;
	case M_SEARCH_SEND: /* Menu "Send" */
		MSearchSend = active;
		break;
	case M_SEARCH_HISTORY: /* Menu "History" */
		MSearchHistory = active;
		break;
	case M_SEARCH_STATUS: /* Menu "Status" */
		MSearchStatus = active;
		break;
	case M_RESULT_DISPLAY: /* Menu "Display Records" */
		MResultDisplay = active;
		if (active == TRUE)
		{
			MFileSave = active;
		}
		break;
	case M_RESULT_STATUS: /* Menu "Status" */
		MResultStatus = active;
		break;
	default:
		xvt_error("MAIN_MENUBAR(): unsupported case");
		break;
	}
    UaEp_MenuState();
}

BOOLEAN UaEp_MenuGetStatus(MENU_TAG tag)
{
	BOOLEAN active;

	switch (tag) 
	{
	/*case M_FILE_NEW: /* Menu "New" */
		active = MFileNew;
		break;
	/*case M_FILE_OPEN: /* Menu "Open" */
		active = MFileOpen;
		break;
	/*case M_FILE_CLOSE: /* Menu "Close" */
		active = MFileClose;
		break;
	case M_FILE_SAVE: /* Menu "Save" */
		active = MFileSave;
		break;
	/*case M_FILE_SAVE_AS: /* Menu "Save As..." */
		active = MFileSaveAs;
		break;
	/*case M_FILE_REVERT: /* Menu "Revert" */
		active = MFileRevert;
		break;
	/*case M_FILE_PG_SETUP: /* Menu "Page Setup..." */
		active = MFilePgSetup;
		break;
	/*case M_FILE_PRINT: /* Menu "Print" */
		active = MFilePrint;
		break;
	/*case M_EDIT_UNDO: /* Menu "Undo" */
		active = MEditUndo;
		break;
	case M_EDIT_CUT: /* Menu "Cut" */
		active = MEditCut;
		break;
	case M_EDIT_COPY: /* Menu "Copy" */
		active = MEditCopy;
		break;
	case M_EDIT_PASTE: /* Menu "Paste" */
		active = MEditPaste;
		break;
	case M_EDIT_CLEAR: /* Menu "Clear" */
		active = MEditClear;
		break;
	case M_EDIT_SEL_ALL: /* Menu "Select All" */
		active = MEditSelAll;
		break;
	case M_EDIT_CLIPBOARD: /* Menu "Show Clipboard" */
		active = MEditClipboard;
		break;
	case M_CONNECT_BREAK: /* Menu "Break" */
		active = MConnectBreak;
		break;
	case M_CONNECT_STATUS: /* Menu "Status" */
		active = MConnectStatus;
		break;
	case M_SEARCH_CLEAR: /* Menu "Clear" */
		active = MSearchClear;
		break;
	case M_SEARCH_DATABASE: /* Menu "Database" */
		active = MSearchDatabase;
		break;
	case M_SEARCH_SEND: /* Menu "Send" */
		active = MSearchSend;
		break;
	case M_SEARCH_HISTORY: /* Menu "History" */
		active = MSearchHistory;
		break;
	case M_SEARCH_STATUS: /* Menu "Status" */
		active = MSearchStatus;
		break;
	case M_RESULT_DISPLAY: /* Menu "Display Records" */
		active = MResultDisplay;
		break;
	case M_RESULT_STATUS: /* Menu "Status" */
		active = MResultStatus;
		break;
	case M_FILE_QUIT:
	case M_FILE_ABOUT_IR: /* Menu "About..." */
	case M_CONNECT_MAKE: /* Menu "Make" */
	case M_SEARCH_QUERY: /* Menu "Query" */
		/* Are Always active. */
		active = TRUE;
		break;
	default:
		xvt_error("MAIN_MENUBAR(): unsupported case");
		active = FALSE;
		break;
	}
	return(active);
}

void UaEp_MenuState(void)
{
	/* Nologner used. may be used in the future.
	 * win_menu_enable(TASK_WIN, M_FILE_NEW, MFileNew);
	 * win_menu_enable(TASK_WIN, M_FILE_OPEN, MFileOpen);
	 * win_menu_enable(TASK_WIN, M_FILE_CLOSE, MFileClose);
	 * win_menu_enable(TASK_WIN, M_FILE_SAVE_AS, MFileSaveAs);
	 * win_menu_enable(TASK_WIN, M_FILE_REVERT, MFileRevert);
	 * win_menu_enable(TASK_WIN, M_FILE_PG_SETUP, MFilePgSetup);
	 * win_menu_enable(TASK_WIN, M_FILE_PRINT, MFilePrint);
	 */
	win_menu_enable(TASK_WIN, M_FILE_SAVE, MFileSave);
	win_menu_enable(TASK_WIN, M_EDIT_UNDO, MEditUndo);
	win_menu_enable(TASK_WIN, M_EDIT_CUT, MEditCut);
	win_menu_enable(TASK_WIN, M_EDIT_COPY, MEditCopy);
	win_menu_enable(TASK_WIN, M_EDIT_PASTE, MEditPaste);
	win_menu_enable(TASK_WIN, M_EDIT_CLEAR, MEditClear);
	win_menu_enable(TASK_WIN, M_EDIT_SEL_ALL, MEditSelAll);
	win_menu_enable(TASK_WIN, M_EDIT_CLIPBOARD, MEditClipboard);
	win_menu_enable(TASK_WIN, M_CONNECT_BREAK, MConnectBreak);
	win_menu_enable(TASK_WIN, M_CONNECT_STATUS, MConnectStatus);
	win_menu_enable(TASK_WIN, M_SEARCH_DATABASE, MSearchDatabase);
	win_menu_enable(TASK_WIN, M_SEARCH_SEND, MSearchSend);
	win_menu_enable(TASK_WIN, M_SEARCH_CLEAR, MSearchClear);
	win_menu_enable(TASK_WIN, M_SEARCH_HISTORY, MSearchHistory);
	win_menu_enable(TASK_WIN, M_SEARCH_STATUS, MSearchStatus);
	win_menu_enable(TASK_WIN, M_RESULT_DISPLAY, MResultDisplay);
	win_menu_enable(TASK_WIN, M_RESULT_STATUS, MResultStatus);
}
