
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ua.h>
#include <resc.h>
#include <ua/ua_param.h>
#include <xvt.h>
#include <uaep.h>
#include <debug.h>
#include <resc.h>
#include <resclist.h>

IR_STATUS UaEp_WinNewData(WINDOW win)
{
	IRO_WORK_SPACE_REQ	wsReq;
	IRO_WORK_SPACE_REPLY	wsRep;
	UARDC_RECORD_STRUCT  	rcd;		/* Record to format. */

	IR_STATUS   		returnCode;
	UAEP_WINDOW 		*info;
	SHORT index;
	IR_RECORD_STRUCT 	*rawRecs;
	IR_DB_RECORD		*dbRec;
	IR_DIAG_RECORD		*diagRec;
	UARDC_DISPLAY_STRUCT 	*tmpDisplay;
	CHAR			tmpBuf2[IR_LEN_50];
	CHAR			tmpBuf[IR_LEN_500];
	CHAR			*t;


	RESC_ASN_NO_MALLOC(&wsReq);
	RESC_ASN_NO_MALLOC(&wsRep);
	RESC_ASN_NO_MALLOC(&wsReq.recList);
	RESC_LIST_INIT(&wsReq.recList);

	info = UaEp_GetWinInfo(win);

CS_LOG_PRINTS(IR_FILE, "Getting workspace info\n");
	wsReq.wsId = info->wsId;
	returnCode = IRO_WorkSpaceInfo(UaEp_InfoUserId(),
	    IRO_WS_INFO_REC_ALL, &wsReq, &wsRep);
	if (returnCode != IR_SUCCESS)
	{
		xvt_error("Unable to get records to display");
		return(returnCode);
	}

	if (RESC_LIST_SIZE(wsRep.records) == 0)
	{
		close_window(win);
		return(IR_FAIL);
	}
	if (info->nRec != 0)
	{
		/* Free up previous records */
		UaEp_DeleteRecList(info->recs, info->nRec);
		info->recs = (UAEP_DISPLAY_REC *)NULL;
	}
	/*
	 * must allocate space for the records
	 */
	info->recs = (UAEP_DISPLAY_REC *)RescIR_Malloc(IR_FILE,
	    (sizeof(UAEP_DISPLAY_REC) * 
	    RESC_LIST_SIZE(wsRep.records)));
	info->topline = 0;
	info->horg = 0;
	info->maxlines = RESC_LIST_SIZE(wsRep.records);
	info->maxwidth = 0;
	info->nRec = 0;

	if (info->recs == (UAEP_DISPLAY_REC *)NULL)
	{
		xvt_fatal("Memory Error allocating space for records");
		return(IR_FAIL);
	}

	/* Convert each marc record to a display format */
	index = 0;
	RESC_LIST_SET_FIRST(wsRep.records);
	for (index = 0; 
	    index < RESC_LIST_SIZE(wsRep.records);
	    index++)
	{
	

		rawRecs = RESC_LIST_GET(wsRep.records);

		if ((tmpDisplay = (UARDC_DISPLAY_STRUCT *)RescIR_Zmalloc(
		    IR_FILE,
		    sizeof(UARDC_DISPLAY_STRUCT)))
		    == (UARDC_DISPLAY_STRUCT *)NULL)
		{
			CS_LOG_PRINTV(IR_FILE, "Mem error on record %d\n",
			    index);
			xvt_fatal("Memory Fault Allocating display");
			return(IR_FAIL);
		}

		switch(rawRecs->recordType)
		{
		case IR_REC_TYPE_SURR_DIAG:
		case IR_REC_TYPE_DB_NON_SURR:
			diagRec = &rawRecs->diagRecord;
			UaHec_GetDiagMsg(MSG_DIAG_TYPE_BIB1, 
			    diagRec->condition,
			    &t);
			strcpy(tmpBuf, t);
			tmpDisplay->displayLines = (CHAR **)RescIR_Malloc(
			    IR_FILE, sizeof(CHAR *));
			tmpDisplay->displayLines[0] = (CHAR *)RescIR_Malloc(
			    IR_FILE, strlen(tmpBuf)+1);
			strcpy(tmpDisplay->displayLines[0], tmpBuf);
			tmpDisplay->lines = 1;
			break;
		case IR_REC_TYPE_DB_RECORD:
			dbRec = &rawRecs->dbRecord;
			if (dbRec->dbRecSyntax.sPtr)
			{
				if ((RescASN_CmpOIToStrOI(IR_FILE, 
				    &dbRec->dbRecSyntax, OID_REC_SYN_USMARC)
				    == IR_SUCCESS) ||
				    (RescASN_CmpOIToStrOI(IR_FILE, 
				    &dbRec->dbRecSyntax, OID_REC_SYN_CANMARC)
				    == IR_SUCCESS) ||
				    (RescASN_CmpOIToStrOI(IR_FILE, 
				    &dbRec->dbRecSyntax, OID_REC_SYN_UKMARC)
				    == IR_SUCCESS))
				{
					rcd.usmarc = (UARDC_USMARC_RECORD)dbRec->record.sPtr;
					if (UaRdc_Format(&(info->fmt), &rcd, tmpDisplay)
						!= IR_SUCCESS)
					{
						CS_LOG_PRINTS(IR_FILE, 
						    "memory error decoding marc record\n");
						xvt_fatal("Memory Fault Allocating display");
					}
				}
				else
				{
					VSToStrNullTerm(tmpBuf2, 
					    &dbRec->dbRecSyntax, IR_LEN_50);
					sprintf(tmpBuf, "Cannot decode marc record using object identifier (%s) provided by target", tmpBuf2);
					tmpDisplay->displayLines = 
					    (CHAR **)RescIR_Malloc(
			    		    IR_FILE, sizeof(CHAR *));
					tmpDisplay->displayLines[0] = 
					    (CHAR *)RescIR_Malloc(
			    		    IR_FILE, strlen(tmpBuf)+1);
					strcpy(tmpDisplay->displayLines[0], 
					    tmpBuf);
					tmpDisplay->lines = 1;
				}
			}
			else
			{
				CS_LOG_PRINTS(IR_FILE, "No record syntax\n");
				strcpy(tmpBuf, "No marc record provided by target");
				tmpDisplay->displayLines = 
				    (CHAR **)RescIR_Malloc(
		    		    IR_FILE, sizeof(CHAR *));
				tmpDisplay->displayLines[0] = 
				    (CHAR *)RescIR_Malloc(
		    		    IR_FILE, strlen(tmpBuf)+1);
				strcpy(tmpDisplay->displayLines[0], 
				    tmpBuf);
				tmpDisplay->lines = 1;
			}
			break;
		default:
			CS_LOG_PRINTV(IR_FILE, "Unknown record type %d\n",
			    rawRecs->recordType);
			xvt_note("Unknown record type %d", rawRecs->recordType);
			return(IR_FAIL);
			break;
		}

		if (tmpDisplay->lines == 0)
		{
			tmpDisplay->lines = 1;
			tmpDisplay->displayLines = (CHAR **)RescIR_Malloc(
			    IR_FILE,sizeof(CHAR *));
			tmpDisplay->displayLines[0] = RescIR_StrDup(
			    IR_FILE,
			    "No fields to display");
		}
		info->nlines += tmpDisplay->lines;
		(info->recs[index].d) = tmpDisplay;
		(info->nRec)++;
		RESC_LIST_SET_NEXT(wsRep.records);
	}
	/* Open the gate and let events back into the windows event handler */
	UaEp_CalcDimensions(win);
	UaEp_ScrollSync(win);
	return(IR_SUCCESS);
}

IR_STATUS UaEp_SaveWinData(IRO_WORK_SPACE_ID wsId, char *filename)
{
	IRO_WORK_SPACE_REQ	wsReq;
	IRO_WORK_SPACE_REPLY	wsRep;

	IR_STATUS   		returnCode;
	SHORT index;
	IR_RECORD_STRUCT 	*rawRecs;
	IR_DB_RECORD		*dbRec;
	FILE			*fp;
	CHAR 			*mode;
	size_t			recSize;
	UARDC_USMARC_RECORD	usmarc;

	mode = "w"; /* Allways write. */
	fp = fopen(filename, mode);

	RESC_ASN_NO_MALLOC(&wsReq);
	RESC_ASN_NO_MALLOC(&wsRep);
	RESC_ASN_NO_MALLOC(&wsReq.recList);
	RESC_LIST_INIT(&wsReq.recList);

	wsReq.wsId = wsId;
	index = 0;

	returnCode = IRO_WorkSpaceInfo(UaEp_InfoUserId(),
	    IRO_WS_INFO_REC_ALL, &wsReq, &wsRep);
	if (returnCode != IR_SUCCESS)
	{
		xvt_error("Unable to get records for save");
		fclose(fp);
		return(returnCode);
	}

	index = 0;
	RESC_LIST_SET_FIRST(wsRep.records);
	for (index = 0; 
	    index < RESC_LIST_SIZE(wsRep.records);
	    index++)
	{
	

		rawRecs = RESC_LIST_GET(wsRep.records);

		switch(rawRecs->recordType)
		{
		case IR_REC_TYPE_SURR_DIAG:
		case IR_REC_TYPE_DB_NON_SURR:
			break;
		case IR_REC_TYPE_DB_RECORD:
			dbRec = &rawRecs->dbRecord;
			recSize = (size_t)dbRec->record.len;
			usmarc = (UARDC_USMARC_RECORD)dbRec->record.sPtr;
			fwrite((void *)usmarc, 1, recSize, fp);
			break;
		default:
			CS_LOG_PRINTV(IR_FILE, "Unknown record type %d\n",
			    rawRecs->recordType);
			xvt_note("Unknown record type %d", rawRecs->recordType);
			fclose(fp);
			return(IR_FAIL);
			break;
		}

	}

	fclose(fp);
	return(IR_SUCCESS);
}



IR_USER_ID UaEp_InfoUserId(void)
{
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);

	return (taskwin->userId);
}

void UaEp_SetActiveWs(IRO_WORK_SPACE_ID wsId)
{
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
	taskwin->activeWs = wsId;
}


IRO_WORK_SPACE_ID UaEp_GetActiveWs(void)
{
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
	return(taskwin->activeWs);
}


UAEP_WORK_SPACE_LIST *GetWsList(IRO_WORK_SPACE_ID wsId)
{
	UAEP_WORK_SPACE_LIST *item;
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);

	item = taskwin->wsRoot;
	while ((item != (UAEP_WORK_SPACE_LIST *)NULL) && (item->wsId != wsId))
	{
		item = item->next;
	}
	return(item);
}


#define SEARCH_NAME	"SEARCH_%d"
CHAR *UaEp_GenerateName(IRO_WORK_SPACE_ID wsId)
{
	CHAR namebuf[80];
	sprintf(namebuf, SEARCH_NAME, wsId);
	return(RescIR_StrDup(IR_FILE, namebuf));
}


CHAR *UaEp_GetWsName(IRO_WORK_SPACE_ID wsId)
{
	static CHAR nameBuf[IR_LEN_100];
	CHAR *s;
	UAEP_WORK_SPACE_LIST *item;

	item = GetWsList(wsId);
	if (item == (UAEP_WORK_SPACE_LIST *)NULL)
	{
		/* Use only in emergencies. Logic not present to handle a
		 * name without a window being attached.
	 	 */
		s = UaEp_GenerateName(wsId);
		strcpy(nameBuf, s);
		RescIR_Free(IR_FILE, s);
		
		return(nameBuf);
	}
	return(item->wsName);
}


static UAEP_WORK_SPACE_LIST *newWorkspace( IRO_WORK_SPACE_ID wsId)
{
	UAEP_WORK_SPACE_LIST *newItem;
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);
	taskwin->activeWs = wsId;
	/*
	 * Allocate space for the new workspace information.
	 */
	if ((newItem = (UAEP_WORK_SPACE_LIST *)RescIR_Malloc(IR_FILE,
	    sizeof(UAEP_WORK_SPACE_LIST))) == (UAEP_WORK_SPACE_LIST *)NULL)
	{
		xvt_error("Fatal Error, Unable to create new workspace.");
		return ((UAEP_WORK_SPACE_LIST *)NULL);
	}

	/*
	 * Set the information.
	 */
	newItem->wsId = wsId;
	newItem->win = (WINDOW)NULL;
	newItem->wsName = UaEp_GenerateName(wsId);
	/*
	 * Make the new item point at what the root pointed at and
	 * make the root point at the new item.
	 */
	newItem->next = taskwin->wsRoot;
	taskwin->wsRoot = newItem;
	return (newItem);
}

/* Snafooed this baby from wswindow(Roland) */
#define MAGIC 		25931	/* magic number */

IR_STATUS UaEp_UpdateWin(IRO_WORK_SPACE_ID wsId)
{
	IRO_WORK_SPACE_REQ	wsReq;
	IRO_WORK_SPACE_REPLY	wsRep;
	UAEP_WORK_SPACE_LIST *upItem;
	UAEP_WINDOW   *winData;
	static UARDC_USMARC_FORMAT_LIST usmarcList[5] = {
	    {{"245"}, "Title"},
	    {{"250"}, "Edition"},
	    {{"260"}, "Publication"},
	    {{"300"}, "Description"},
	    {{""}, ""}};

	/* Check for this wsId to already be associated with a window */
	if ((upItem = GetWsList(wsId)) != (UAEP_WORK_SPACE_LIST *)NULL)
	{
		/* Close the window that is already open */
		close_window(upItem->win);
	}

	RESC_ASN_NO_MALLOC(&wsReq);
	RESC_ASN_NO_MALLOC(&wsRep);
	RESC_ASN_NO_MALLOC(&wsReq.recList);
	RESC_LIST_INIT(&wsReq.recList);

CS_LOG_PRINTS(IR_FILE, "Getting workspace info to see if records are required\n");
	wsReq.wsId = wsId;
	if ((IRO_WorkSpaceInfo(UaEp_InfoUserId(),
	    IRO_WS_INFO_REC_ALL, &wsReq, &wsRep) != IR_SUCCESS) ||
	   (RESC_LIST_SIZE(wsRep.records) == 0))
	{
		/* Failed on get or no records to get */
		return(IR_FAIL);
	}

	winData = (UAEP_WINDOW *)RescIR_Zmalloc(IR_FILE,
	    sizeof(UAEP_WINDOW));
	if (winData == NULL)
	{
		CS_LOG_PRINTS(IR_FILE, "malloc error bye bye!\n");
		return(IR_FAIL);
	}

	if ((upItem = newWorkspace(wsId)) == 
	    (UAEP_WORK_SPACE_LIST *)NULL)
	{
		RescIR_Free(IR_FILE, (CHAR *)winData);
		return(IR_FAIL);
	}
	winData->magic = MAGIC;
	winData->p_rcd = (PRINT_RCD *)NULL;
	(void)select_font(FF_SYSTEM, 0, 10, &(winData->font));

	winData->fmt.type = UARDC_TYPE_BRIEF;
	winData->fmt.lineSz = 120;
	winData->title = RescIR_StrDup(IR_FILE, upItem->wsName);
	winData->wsId = wsId;
	winData->fmt.rType = USMARC_REC_TYPE;
	winData->fmt.style = UARDC_STYLE_NONE;
	winData->fmt.type = UARDC_TYPE_BRIEF;
	winData->fmt.trunkAt = 0;
	winData->fmt.l.usmarcList = usmarcList;

	upItem->win = UaEp_InvokeWSWin((long)winData);

	return(UaEp_WinNewData(upItem->win));
	
}

void DeleteWsItem(UAEP_WORK_SPACE_LIST *item)
{
	/*
	 * Possibly want to do other commands at this point to clean up
	 * what the item is pointing at like close the window etc....
	 */
	if ((item)  && (item->wsName))
	{
		(void)RescIR_Free(IR_FILE, (CHAR *)item->wsName);
	}
	
	if (item)
	{
		(void)RescIR_Free(IR_FILE, (CHAR *)item);
	}
}


void UaEp_DelWorkspace(IRO_WORK_SPACE_ID wsId)
{
	UAEP_WORK_SPACE_LIST *curItem;
	UAEP_WORK_SPACE_LIST *prevItem;
	UAEP_SYSTEM_INFO_STRUCT *taskwin;/* Pointer to task win
					  * maintained info */

	taskwin = (UAEP_SYSTEM_INFO_STRUCT *)get_app_data(TASK_WIN);

	prevItem = (UAEP_WORK_SPACE_LIST *)NULL;
	curItem = taskwin->wsRoot;
	while ((curItem != (UAEP_WORK_SPACE_LIST *)NULL) &&
	    (curItem->wsId != wsId))
	{
		prevItem = curItem;
		curItem = curItem->next;
	}

	if (curItem == (UAEP_WORK_SPACE_LIST *)NULL)
	{
		return;
	}

	if (prevItem == (UAEP_WORK_SPACE_LIST *)NULL)
	{
		/* The root is pointing at it. */
		taskwin->wsRoot = curItem->next;
	}
	else
	{
		/* Another item is pointing at it. */
		prevItem->next = curItem->next;
	}

	DeleteWsItem(curItem);
}

