/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 ************************************************************************/

#include <ir.h>
#include <ua.h>
#include <ua/ua_rdc.h>
#include <resc.h>
#include <debug.h>
#include <resclist.h>
#include <xvt.h>
#include <uaep.h>

#define MAGIC 		25931	/* magic number */
#define GRANULE		100	/* allocation granule for lines array */
#define MGN		2	/* margin (pixels) */
#define HINTERVAL	50	/* horizontal scrolling interval (pixels) */


/*
	Creation routine for window WIN_WORKSPACE ("workspace")
*/
WINDOW UaEp_InvokeWSWin(long data)
{
	WINDOW win;
	UAEP_WINDOW *info;


	static WIN_DEF win_def[2];
	static int dh;
	static int dv;
	static RCT rct = {8, 8, 168, 320};

	info = (UAEP_WINDOW *)data;
	dh= (int)get_value(NULL_WIN, ATTR_DOC_STAGGER_HORZ);
	dv= (int)get_value(NULL_WIN, ATTR_DOC_STAGGER_VERT);
	offset_rect(&rct, dh, dv);
	win_def[0].wtype = W_DOC;
	win_def[0].rct = rct;
	win_def[0].text = info->title;
	win_def[0].units = U_PIXELS;
	win_def[0].v.win.flags = (WSF_NO_MENUBAR | WSF_DECORATED |  WSF_ICONIZABLE | WSF_MAXIMIZED);
	win_def[1].wtype = W_NONE;

	if ((win = create_def_window(win_def,
	    TASK_WIN,
	    EM_ALL,
	    win_501_eh,
	    (long)info)) == NULL_WIN)
	{
		xvt_fatal("Cannot create new window. (%d)", get_xvterrno());
	}

	return (win);
}


void UaEp_MouseDouble(WINDOW win, RCT rct, PNT  pnt)
{
	DRAW_CTOOLS curTools;
	DRAW_CTOOLS invTools;
	SHORT index;
	SHORT leading;
	SHORT ascent;
	SHORT descent;
	SHORT offset;
	SHORT client_height;
	UAEP_WINDOW *info;
	UAEP_REC_LOC dRec;

	info = UaEp_GetWinInfo(win);

	client_height = rct.bottom - rct.top;

	(void)win_get_draw_ctools(win, &curTools);
	(void)win_get_draw_ctools(win, &invTools);

	invTools.pen.pat = PAT_HOLLOW;
	invTools.brush.color = COLOR_BLACK;
	invTools.brush.pat = PAT_SOLID;
	invTools.mode = M_XOR;

	win_set_font(win, &info->font, FALSE);

	win_get_font_metrics(win, &leading, &ascent, &descent);

	/*
	 * start with first visible line
	 */
	dRec.rec = info->topRec.rec;

	dRec.line = info->topRec.line;

	for (index = info->topline, offset = leading + ascent;
	    index < info->nlines && offset <= client_height - descent;
	    index++, offset += info->line_height)
	{

		set_rect(&rct,
		    rct.left,
		    offset - ascent,
		    rct.right,
		    (offset + descent));

		if (pt_in_rect(&rct, pnt))
		{
			/* Reverse the colours on the line in question. */
			win_set_draw_ctools(win, &invTools);
			win_draw_rect(win, &rct);
			/*
			 * Display the text in a dialog and put the text
			 * back to the original colour when they are done.
			 */
			UaHec_Message((CHAR *)NULL, 
			    (CHAR *)info->recs[dRec.rec].d->
			    displayLines[dRec.line],
			    TRUE);
			win_draw_rect(win, &rct);
			win_set_draw_ctools(win, &curTools);
			return;
		}
		UaEp_NextRec(&dRec, info);
	}

}

void UaEp_DoUpdate(WINDOW win, RCT rct)
{
	DRAW_CTOOLS drawTools;
	SHORT index;
	SHORT leading;
	SHORT ascent;
	SHORT descent;
	SHORT offset;
	SHORT client_height;
	UAEP_WINDOW *info;
	UAEP_REC_LOC dRec;

	info = UaEp_GetWinInfo(win);

	client_height = rct.bottom - rct.top;

	(void)win_get_draw_ctools(win, &drawTools);

	clear_window(win, drawTools.back_color);
	
	win_set_font(win, &info->font, FALSE);

	win_get_font_metrics(win, &leading, &ascent, &descent);

	/*
	 * start with first visible line
	 */
	dRec.rec = info->topRec.rec;

	dRec.line = info->topRec.line;

	for (index = info->topline, offset = leading + ascent;
	    index < info->nlines && offset <= client_height - descent;
	    index++, offset += info->line_height)
	{



		set_rect(&rct,
		    rct.left,
		    offset - ascent,
		    rct.right,
		    (offset + descent));

		if (needs_update(win, &rct))
		{
			win_draw_text(win,
			    (MGN - info->horg),
			    offset,
			    info->recs[dRec.rec].d->displayLines[dRec.line],
			    -1);
		}
		UaEp_NextRec(&dRec, info);
	}

}

UAEP_WINDOW *UaEp_GetWinInfo(WINDOW win)
{
	UAEP_WINDOW *info = (UAEP_WINDOW *)get_app_data(win);
	assert4(info != NULL && info->magic == MAGIC, "getWinInfo", 7, "Bad DOC data");
	return info;
}

void UaEp_CalcDimensions(WINDOW win)
{
	UAEP_WINDOW *info;
	SHORT index;
	SHORT lineWidth;
	SHORT leading;
	SHORT ascent;
	SHORT descent;

	info = UaEp_GetWinInfo(win);

	win_set_font(win, &info->font, FALSE);
	win_get_font_metrics(win, &leading, &ascent, &descent);
	info->line_height = leading + ascent + descent;
	info->maxwidth = 0;
	for (index = 0; index < info->nlines; index++)
	{
		lineWidth = win_get_text_width(win,
		    info->recs[index].d->displayLines[0],
		    -1);
		if (lineWidth > info->maxwidth)
		{
			info->maxwidth = lineWidth;
		}
	}
	info->maxwidth += MGN;
}


void UaEp_DeleteWs(UAEP_WINDOW *info)
{

	if (info == NULL)
	{
		return;
	}

	/* Dispose of text */
	UaEp_DeleteRecList(info->recs, info->nRec);

	if (info->title)
	{
		RescIR_Free(IR_FILE, (CHAR *)info->title);
	}

	/* Delete the workspace from the list */
	UaEp_DelWorkspace(info->wsId);

	/*
	 * dispose of print record
	 */
	if (info->p_rcd != NULL)
	{
		free_print_rcd(info->p_rcd);
	}
	(void)RescIR_Free(IR_FILE, (CHAR *)info);
}

void UaEp_DeleteRecList(UAEP_DISPLAY_REC *recList, SHORT nRecs)
{

	SHORT index;
	SHORT recIndex;

	/*
	 *dispose of text
	 */
	if (recList != (UAEP_DISPLAY_REC *)NULL)
	{
		for (index = 0; index < nRecs; index++)
		{
			for (recIndex = 0;
			    recIndex < (SHORT)recList[index].d->lines;
			    recIndex++)
			{
				if (recList[index].d->
				    displayLines[recIndex]
				    != NULL)
				{
					(void)RescIR_Free(IR_FILE,
					    recList[index].d->
					    displayLines[recIndex]);
				}
			}
			(void)RescIR_Free(IR_FILE,
			    (CHAR *)recList[index].d);
		}
		if (recList)
		{
			(void)RescIR_Free(IR_FILE, (CHAR *)recList);
		}
	}
}

void UaEp_GetPrinterSize(PRINT_RCD *pr, long *height, long *width)
{
#if (XVT_OS == XVT_OS_MAC) || (XVTWS == PMWS)
	(void)xvt_escape(XVT_ESC_GET_PRINTER_INFO, (PRINT_RCD far *)pr,
		(long far *)height, (long far*)width, NULL, NULL);
#else
	*width = get_value(NULL_WIN, ATTR_PRINTER_WIDTH);
	*height = get_value(NULL_WIN, ATTR_PRINTER_HEIGHT);
#endif
}

void UaEp_NextRec(UAEP_REC_LOC *rec, UAEP_WINDOW *info)
{
	SHORT nextrec;
	SHORT rline;


	nextrec = rec->rec;
	rline = rec->line;
	rline++;
	if (rline >= (SHORT)info->recs[nextrec].d->lines)
	{
		nextrec++;
		rline = 0;
	}
	rec->rec = nextrec;
	rec->line = rline;
}

void UaEp_SetTopRec(UAEP_WINDOW *info)
{
	SHORT	curLine;    /* Current line index. */
	SHORT rec;
	SHORT rline;

	curLine = 0;
	rec = 0;
	rline = 0;
	while ((curLine < info->topline) && (rec < info->nRec))
	{
		curLine++;
		rline++;
		if (rline >= (SHORT)info->recs[rec].d->lines)
		{
			rec++;
			rline = 0;
		}
	}
	info->topRec.rec = rec;
	info->topRec.line = rline;
}


BOOLEAN UaEp_PrintDocument(WINDOW win)
{
	RCT rct;
	WINDOW pwin;
	long printer_width, printer_height;
	int leading, ascent, descent, lines_per_page;
	UAEP_WINDOW pd;
	UAEP_WINDOW *info;

	info = UaEp_GetWinInfo(win);

	print_begin();
	if (info->p_rcd == NULL ||
	  (pwin = new_print_window(info->p_rcd, info->title)) == NULL_WIN)
	{
		return TRUE; /* user cancelled -- no actual error */
	}

	pd = *info;
	pd.horg = 0;
	set_app_data(pwin, PTR_LONG(&pd));
	win_set_font(pwin, &pd.font, FALSE);
	win_get_font_metrics(pwin, &leading, &ascent, &descent);
	pd.line_height = leading + ascent + descent;
	UaEp_GetPrinterSize(pd.p_rcd, &printer_height, &printer_width);
	set_rect(&rct, 0, 0, (int)printer_width, (int)printer_height);
	lines_per_page = (int)(printer_height / pd.line_height);

	for (pd.topline = 0;
	    pd.topline < pd.nlines;
	    pd.topline += lines_per_page)
	{
		UaEp_SetTopRec(&pd);
		if (!start_page(pd.p_rcd))
		{
			break;
		}
		while (next_band() != NULL)
		{
			UaEp_DoUpdate(pwin, rct);
		}
		if (!finish_page(pd.p_rcd))
		{
			break;
		}
	}
	close_print_window(pwin, pd.p_rcd);
	return TRUE;
}

void UaEp_ScrollSync(WINDOW win)
{
	RCT rct;
	UAEP_WINDOW *info;
	SHORT vis_lines;
	int hprop;
	int vprop;


	info = UaEp_GetWinInfo(win);
	get_client_rect(win, &rct);
	vis_lines = rct.bottom / info->line_height;
	set_scroll_range(win, VSCROLL, 0, info->nlines);
	set_scroll_range(win, HSCROLL, 0, info->maxwidth);
	hprop = min(rct.right, (int)info->maxwidth);
	vprop = min((int)vis_lines, (int)info->nlines);

	/* The following 2 "if" statements keep bootom line or right edge of
	 * document glued to end/edge of window when resized, as long as
	 * there is stuff to scroll in from top or left.
	 */
	if (info->topline + vis_lines >= info->nlines)
	{
		UaEp_DoScroll(win,
		    VSCROLL,
		    SC_THUMB,
		    max(0, info->nlines - vis_lines));
	}
	if (info->horg + rct.right >= info->maxwidth)
	{
		UaEp_DoScroll(win,
		    HSCROLL,
		    SC_THUMB,
		    max(0, info->maxwidth - rct.right));
	}

	set_scroll_proportion(win, VSCROLL, vprop);
	set_scroll_proportion(win, HSCROLL, hprop);

	set_scroll_pos(win,
	    VSCROLL,
	    min(info->nlines - vprop, info->topline));

	set_scroll_pos(win,
	    HSCROLL,
	    min(info->maxwidth - hprop, info->horg));
}

void UaEp_DoScroll(WINDOW win,
    SCROLL_TYPE type,
    SCROLL_CONTROL what,
    short pos)
{
	RCT rct;
	int old_horg;
	int old_top;
	int dh;
	int dv;
	int page;
	int maxorg;
	int bottom;
	UAEP_WINDOW *info;

	info = UaEp_GetWinInfo(win);

	update_window(win);
	old_horg = info->horg;
	old_top = info->topline;
	get_client_rect(win, &rct);
	page = rct.bottom / info->line_height;

	switch (type)
	{
	case HSCROLL:
		maxorg = max(0, info->maxwidth - rct.right);
		switch (what)
		{
		case SC_LINE_UP:
			info->horg = max(0, old_horg - HINTERVAL);
			break;

		case SC_LINE_DOWN:
			if (old_horg < maxorg)
			{
				info->horg = min(maxorg, old_horg + HINTERVAL);
			}
			break;

		case SC_PAGE_UP:
			info->horg = max(0, old_horg - rct.right);
			break;

		case SC_PAGE_DOWN:
			info->horg = min(maxorg, old_horg + rct.right);
			break;

		case SC_THUMB:
			info->horg = pos;
			break;
		default:
			break;
		}
		break;


	case VSCROLL:
		maxorg = max(0, info->nlines - page);
		switch (what)
		{
		case SC_LINE_UP:
			info->topline = max(0, old_top - 1);
			break;

		case SC_LINE_DOWN:
			if (old_top < maxorg)
			{
				info->topline = min(maxorg, old_top + 1);
			}
			break;

		case SC_PAGE_UP:
			info->topline = max(0, old_top - page);
			break;

		case SC_PAGE_DOWN:
			info->topline = min(maxorg, old_top + page);
			break;

		case SC_THUMB:
			info->topline = pos;
			break;
		default:
			break;
		}
	default:
		break;
	}

	UaEp_SetTopRec(info);

	dh = old_horg - info->horg;
	dv = (old_top - info->topline) * info->line_height;

	if (dh != 0 || dv != 0)
	{
		/*
		 * scroll just integral lines
		 */
		bottom = rct.bottom;
		rct.bottom = page * info->line_height;
		win_scroll_rect(win, &rct, dh, dv);
		if (dv != 0)
		{
			set_scroll_pos(win,
			    VSCROLL,
			    min((int)maxorg, (int)info->topline));
		}

		if (dh != 0)
		{
			set_scroll_pos(win,
			    HSCROLL,
			    min((int)maxorg, (int)info->horg));
		}

		/*
		 * invalidate partial last line
		 */
		if (dv != 0 && bottom != rct.bottom)
		{
			bottom = rct.bottom;
			rct.top = rct.bottom;
			rct.bottom = bottom;
			invalidate_rect(win, &rct);
		}
	}
}
