/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Help and Error Control User Message 
 *
 * DESCRIPTION:
 *	This unit contains the functions needed to convert internal 
 *	codes to user displayable messages.
 *
 * CONTENTS:
 *	UaHec_Initialize
 *	UaHec_UserMsg
 *	UaHec_Message
 *
 * NOTES:
 *	This CSU may only be invoked from with in the User Application
 * 	Layer of the IR Kernel-Origin
 *
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
/***************************************************************************/

#include <ir.h>
#include <ir_api.h>
#include <uaep.h>
#include <ua.h>
#include <ua/uaepmsg.h>
#include <resc.h>

#ifndef XVT_MSDOS
#include <stdio.h>
#include <string.h>
extern FILE 	*CS_LogFp;
#endif

#define IR_SYSTEM_MSG	"CanSearch System Message"
typedef struct
{
	UAHEC_MSG_ID	msgId;		/* unique identifier */
	BOOLEAN		option;		/* option control */
	CHAR		*string;	/* Displayable string */
} MSG_TABLE;

static MSG_TABLE	assocTable[]=
{
	{MSG_ID_AC_CONNECTING, TRUE, "Connecting"},
	{MSG_ID_AC_CONNECTED, TRUE, "Connected"},
	{MSG_ID_AC_CONN_REFUSED, TRUE, "Connection Refused"},
	{MSG_ID_AC_UNKNOWN_HOST, TRUE, "Couldn't Resolve host name/addr"},
	{MSG_ID_AC_INIT_COMPLETE, TRUE, "IR initialization complete"},
	{MSG_ID_AC_INIT_FAIL, TRUE, "IR initialization failed"},
	{MSG_ID_AC_CONN_REL, TRUE, "Connection released"},
	{MSG_ID_AC_CONN_REL_FAIL, TRUE, "Connection release attempt failed"},
	{MSG_ID_AC_ABORT, TRUE, "Connection aborted"},
	{MSG_ID_AC_ABORT_FAIL, TRUE, "Connection abort attempt failed"},
	{MSG_ID_AC_TIMEOUT, TRUE, "Connection has timed out"},
	{MSG_ID_AC_PROTOCOL_ERROR, TRUE, "Protocol rrror detected on connection"},
	{MSG_ID_INVALID, TRUE, "Unknown Display Msg Id"}
};

#define CONNECT_TYPE 0
#define RELEASE_TYPE 1
#define ABORT_TYPE 2
#define INIT_TYPE 3


static MSG_TABLE	epTable[]=
{
	{MSG_ID_EP_LOGIN_REQD, TRUE, "You must enter both \"login name\" and \"password\""},
	{MSG_ID_INVALID, TRUE, "Unknown Display Msg Id"}
};
static BOOLEAN UAHEC_DisplayMsg;
static BOOLEAN UAHEC_DisplayErr;

/****************************************************************************
 * FUNCTION: UaHec_Initialize
 *
 * PURPOSE:
 *	This function will initialize the display message table.
 *
 * INPUT:
 *	NONE.
 *
 * OUTPUT:
 *	NONE.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_Initialize()
{
	UaHec_InitAssocNameTbl();
	UAHEC_DisplayMsg = TRUE;
	UAHEC_DisplayErr = TRUE;
	return;

}

void UaHec_ToggleDispMsg(SHORT type, BOOLEAN use)
{
	switch (type)
	{
	case CONNECT_TYPE:
		assocTable[0].option = use;
		assocTable[1].option = use;
		break;

	case INIT_TYPE:
		assocTable[4].option = use;
		assocTable[5].option = use;
		break;
	case RELEASE_TYPE:
		assocTable[6].option = use;
		assocTable[7].option = use;
		break;
	case ABORT_TYPE:
		assocTable[8].option = use;
		assocTable[9].option = use;
		break;
	}
}

void UaHec_ToggleDispErr()
{
	if (UAHEC_DisplayErr == TRUE)
	{
		UAHEC_DisplayErr = FALSE;
	}
	else
	{
		UAHEC_DisplayErr = TRUE;
	}
}

BOOLEAN	UaHec_DispMsg()
{
	return(UAHEC_DisplayMsg);
}
BOOLEAN UaHec_DispErr()
{
	return(UAHEC_DisplayErr);
}
/****************************************************************************
 * FUNCTION: UaHec_UserMsg
 *
 * PURPOSE:
 *	This function is invoked by other ua units to write messages to
 *	the user's window.
 *
 * INPUT:
 *	(char *)source			- IR_FILE
 *	UAHEC_MSG_TYPE			- message type
 *	UAHEC_MSG_ID			- msg id
 *	(char *)strData			- associated data.
 *	USHORT iData			- assocId
 *
 *
 * OUTPUT:
 *	NONE.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_UserMsg(
	CHAR *source, 
	UAHEC_MSG_TYPE msgType,
    	UAHEC_MSG_ID msgId, 
	CHAR *strData, 
	USHORT iData)
{

	MSG_TABLE	*tbl;
	CHAR		*s;
	CHAR		*src;
	SHORT		i;
	CHAR		*typeName;
	VISIBLE_STRING	*targetName;
	CHAR		targetNameStr[100];
	CHAR		bigBuf[IR_LEN_1000];

	targetName = (VISIBLE_STRING *)NULL;
	if (!UaHec_DispMsg())
	{
		return;
	}

	if (source == (CHAR *)NULL)
	{
		src = "unknown source";
	}
	else
	{
		src = source;
	}
	if (strData == (CHAR *)NULL)
	{
		s = "";
	}
	else
	{
		s = strData;
	}

	strcpy(targetNameStr,"");
	if ((msgType == UAEP_MSG_TYPE_AC) &&
	    (iData != 0))
	{
		/* Get a display string for the associated target */
		targetName = UaHec_GetAssocName((IR_ASSOC_ID)iData);
		if (targetName != NULL)
		{
			VSToStrNullTerm(targetNameStr, targetName, 100);
			RescASN_DestroyVS(IR_FILE, targetName);
		}
	}

	switch(msgType)
	{
	case UAEP_MSG_TYPE_AC:
		tbl = assocTable;
		typeName = "Connection";
		break;

	case UAEP_MSG_TYPE_EP:
		tbl = epTable;
		typeName = "Event Processing";
		break;
	default:
		sprintf(bigBuf,"%s: Unknown Message Type %d, msgId = %d, string Data = %s, integer data = %d. On target: %s",
			src, msgType, msgId, s, iData, targetNameStr);
		UaHec_Message(IR_SYSTEM_MSG, bigBuf, FALSE);
		return;
	}

	i = 0;
	while ((tbl[i].msgId != MSG_ID_INVALID) &&
	    (tbl[i].msgId != msgId))
	{
		i++;
	}

	if (tbl[i].msgId == MSG_ID_INVALID)
	{
		sprintf(bigBuf,
	            "%s: %s, Unknown Message Id %d, string = %s,"
		    " idata = %d on target %s",
		    src, typeName, msgId, s, iData, targetNameStr);
		UaHec_Message(IR_SYSTEM_MSG, bigBuf, FALSE);
	}
	else if (tbl[i].option == TRUE)
	{
		sprintf(bigBuf, "%s: %s, %s, %s, %d on target: %s", src, typeName, tbl[i].string, s,
		    iData, targetNameStr);
		UaHec_Message(IR_SYSTEM_MSG, bigBuf, FALSE);
	}
}

/****************************************************************************
 * FUNCTION: UaHec_Message
 *
 * PURPOSE:
 *	This function is invoked by other ua units to write messages to
 *	the user's window.
 *
 * INPUT:
 *	(CHAR *)title			- Title of message window
 *					  set to (CHAR *)NULL if you want
 *					  "CANSEARCH"
 *	(CHAR *)message			- Null terminated string to display.
 *
 *
 * OUTPUT:
 *	NONE.
 *
 * NOTES:
 *	Nothing to note as of yet.
 ***************************************************************************/

void UaHec_Message(CHAR *title, CHAR *msg, BOOLEAN modal)
{
	static UAEP_DIALOG message;

	memset(&message, (char)0, sizeof(UAEP_DIALOG));
	message.data.show.title = title;
	message.data.show.message = msg;
	
	if (modal == TRUE)
	{
		if (!create_res_dialog(WD_MODAL,
	    	    DLG_FULL_RECORD, EM_ALL, dlg_112_eh, 
	    	    (long)&message))
		{
			xvt_error("Can't open message dialog");
		}
	}
	else
	{

		if (!create_res_dialog(WD_MODELESS,
	    	    DLG_FULL_RECORD, EM_ALL, dlg_112_eh, 
	    	    (long)&message))
		{
			xvt_error("Can't open message dialog");
		}
	}
}

#define FR_LINE_SIZE  60

char *UaHec_get_next_segment(char *buffer, int *offset, BOOLEAN modal)
{
	static char outbuff[FR_LINE_SIZE +1];
	
	int counter;
	
	counter = FR_LINE_SIZE;
	if (counter > (int)strlen(buffer))
	{
		strcpy(outbuff, buffer);
		*offset = -1;
		return(outbuff);
	}

	while (!isspace((int)buffer[counter] ) && (counter > 0))
	{
		counter--;
	}
	if (counter == 0)
	{
		counter = FR_LINE_SIZE;
	}
	*offset = counter;
	strncpy(outbuff, buffer, counter);
	outbuff[counter] = (char)NULL;
	
	while((*offset < (int)strlen(buffer))
	    && 
	    (isspace((int)buffer[*offset]))) 
	{
		(*offset)++;
	}
	if (*offset > (int)strlen(buffer))
	{
		*offset = -1;
	}
	return (outbuff);
}
