
/************************************************************************
 *
 * SR/Z3950 application software version 1.0, August 1993.
 *
 * (C) Software Kinetics Ltd.
 *
 * Software Kinetics Ltd. has granted a licence of this software to the
 * National Library of Canada.
 *
 * The National Library of Canada grants to the user a royalty-free
 * non-exclusive licence in perpetuity to use, modify, improve, reproduce
 * and sell the software.
 *
 * The software is provided to the user "as is" without warranty of any
 * kind. Neither Software Kinetics Ltd. nor the National Library of Canada
 * warrants, guarantees or makes any representations regarding the use,
 * or the results of use of the software in terms of accuracy, reliability,
 * or otherwise. The entire risk as to results and performance of the
 * software is assumed by the user. Neither Software Kinetics Ltd. nor the
 * National Library of Canada shall be liable for any direct, indirect,
 * consequential or incidental damages arising out of the use of the
 * software.
 *
 ************************************************************************/

/****************************************************************************
 * UNIT:  Record Display Result Dialog control
 *
 * DESCRIPTION:
 *
 * CONTENTS:
 *      UaRdc_DRDlgInit
 *      UaRdc_DRDlgGetSelect
 *
 * NOTES:
 *      This CSU may only be invoked from with in the User Application
 *      Layer of the IR Kernel-Origin
 *
 * $Log:
 * Revision 1.1  1993/09/16  09:03:27  irorigin
 * Baseline for Release 1.0.
 *
 *
 *
/***************************************************************************/

#include <ir.h>
#include <uaep.h>
#include <ir_api.h>
#include <ua.h>
#include <resc.h>
#include <debug.h>
#include <resclist.h>

void UaRdc_DRDlgHandleList(WINDOW win , SLIST dispList, UAEP_DIALOG *info);
void UaRdc_DRDlgGetSelect(WINDOW win , SLIST dispList, UAEP_DIALOG *info);
static IR_LIST  Rdc_wsList;

/****************************************************************************
 * FUNCTION: UaRdc_DRInit
 *
 * PURPOSE:
 *
 * INPUT:
 *
 * OUTPUT:
 *
 * NOTES:
 *      Nothing to note as of yet.
 ***************************************************************************/

IR_STATUS UaRdc_DRDlgInit(WINDOW win, SLIST dispList)
{
	IRO_WORK_SPACE_REQ      req;
	IRO_WORK_SPACE_REPLY    reply;
	IRO_WS_HEADER_INFO      *wsInfo;
	EP_DLG_EDIT_TYPE        listData;
	CHAR                    tmpBuf[IR_LEN_100];
	SHORT                   wsNum;
	UAEP_DIALOG             *info;
	VISIBLE_STRING		*targetName;
	CHAR			targetNameStr[IR_LEN_100];
	WINDOW			listWin;

	info = (UAEP_DIALOG *)get_app_data(win);

	RESC_ASN_NO_MALLOC(&req);
	RESC_ASN_NO_MALLOC(&reply);


	if (IRO_WorkSpaceInfo(UaEp_InfoUserId(), IRO_WS_INFO_GEN_ALL,
	    &req, &reply) == IR_FAIL)
	{
		UaEp_MenuSetStatus(M_RESULT_DISPLAY, FALSE);
		return(IR_FAIL);
	}


	/* Initialize the workspace list */
	RESC_ASN_NO_MALLOC(&Rdc_wsList);
	RESC_LIST_INIT(&Rdc_wsList);

	/* Copy the work space header info list retrieved from IRO */
	if (!reply.genInfo)
	{
		UaEp_MenuSetStatus(M_RESULT_DISPLAY, FALSE);
		return(IR_FAIL);
	}
	
	RescList_Copy(IR_FILE, &Rdc_wsList, &reply.genInfo->wsHeaderInfo);
	
	RESC_LIST_SET_FIRST(&Rdc_wsList);
	for (wsNum = 0; wsNum < RESC_LIST_SIZE(&Rdc_wsList); wsNum++)
	{
		wsInfo = RESC_LIST_GET(&Rdc_wsList);
		

		targetName = (VISIBLE_STRING *)NULL;
		strcpy(targetNameStr,"");
		/* Get a display string for the associated target */
		targetName = UaHec_GetAssocName(wsInfo->assocId);
		if (targetName != NULL)
		{
			VSToStrNullTerm(targetNameStr, targetName, IR_LEN_100);
			RescASN_DestroyVS(IR_FILE, targetName);
		}

		VSToStrNullTerm(tmpBuf, &wsInfo->name, IR_LEN_50);

		if (strlen(targetNameStr) > 0)
		{
			sprintf(listData, "%1ld/%1ld SEARCH_%1d,%s, %s", 
		    	    wsInfo->totalTargetRecords, wsInfo->numWSRecords,
		    	    wsInfo->wsId, tmpBuf, targetNameStr);
		}
		else
		{
			sprintf(listData, "%1ld/%1ld SEARCH_%1d,%s", 
			    wsInfo->totalTargetRecords, wsInfo->numWSRecords,
			    wsInfo->wsId, tmpBuf);
		}
		/* Add entry to list */
		slist_add(dispList,
		    (SLIST_ELT)NULL,
		    (char *)listData,
		    (long)wsNum);
		/* Get next ws */
		RESC_LIST_SET_NEXT(&Rdc_wsList);
	}
	if (wsNum == 0)
	{
		UaEp_MenuSetStatus(M_RESULT_DISPLAY, FALSE);
		return(IR_FAIL);
	}
	listWin = get_ctl_window(win, RD_GET_LIST);
	win_list_add(listWin, -1, (STR)dispList);
	set_front_window(listWin);
	win_list_set_sel(listWin, 0, TRUE);

	UaRdc_DRDlgHandleList(listWin , dispList, info);

	return(IR_SUCCESS);
}

void UaRdc_DRDlgHandleList(WINDOW win , SLIST dispList, UAEP_DIALOG *info)
{
	BOOLEAN delRemote;
	BOOLEAN delLocal;
	BOOLEAN fetchRemote;
	BOOLEAN fetchLocal;
	IRO_WS_HEADER_INFO      *wsInfo;
	WINDOW                  ctlWin;
	WINDOW                  targetWin;

	delRemote = FALSE;
	delLocal = TRUE;
	fetchRemote = FALSE;
	fetchLocal = FALSE;

	/* Get the selected list item */
	UaRdc_DRDlgGetSelect(win, dispList, info);

	if (info->data.display.wsId != IR_UNKNOWN_ID_TYPE)
	{

		if (info->data.display.wsHdrInfo)
		{
			wsInfo = info->data.display.wsHdrInfo;

			if ((wsInfo->targetResultsDeleted == FALSE) &&
			    (wsInfo->deleteSrvcSupported == TRUE) &&
			    (wsInfo->deleting == FALSE))
			{
				delRemote = TRUE;
			}
			if ((wsInfo->targetResultsDeleted == FALSE) &&
			    (wsInfo->presentSrvcSupported == TRUE) &&
			    (wsInfo->presenting == FALSE))
			{
				fetchRemote = TRUE;
			}

			if (wsInfo->numWSRecords > 0)
			{
				fetchLocal = TRUE;
			}
		}

	}

	/* Set the buttons according to the boolean indicators */
	targetWin = get_parent(win);

	ctlWin = get_ctl_window(targetWin, RD_PB_DELETE_LOCAL);
	enable_window(ctlWin, delLocal);

	ctlWin = get_ctl_window(targetWin, RD_OK);
	enable_window(ctlWin, fetchLocal);

	ctlWin = get_ctl_window(targetWin, RD_PB_DELETE_REMOTE);
	enable_window(ctlWin, delRemote);

	ctlWin = get_ctl_window(targetWin, RD_PB_FETCH_REMOTE);
	enable_window(ctlWin, fetchRemote);
}

void UaRdc_DRDlgGetSelect(WINDOW win , SLIST dispList, UAEP_DIALOG *info)
{
	long data;
	IRO_WS_HEADER_INFO      *wsInfo;

	if (EpDialog_GetSelListData(win, dispList, &data) != IR_SUCCESS)
	{
		info->data.display.wsId = IR_UNKNOWN_ID_TYPE;
		info->data.display.wsHdrInfo = (IRO_WS_HEADER_INFO *)NULL;
	}
	else
	{
		RESC_LIST_SET_CUR(&Rdc_wsList, (SHORT)data);
		wsInfo = RESC_LIST_GET(&Rdc_wsList);
		info->data.display.wsId = wsInfo->wsId;
		info->data.display.assocId = wsInfo->assocId;
		info->data.display.numRecOnTarget = 
		    wsInfo->totalTargetRecords;
		info->data.display.nextPosOnTarget = 
		    wsInfo->nextTargetRec;
		info->data.display.numRecInWs = 
		    wsInfo->numWSRecords;
		info->data.display.wsHdrInfo = wsInfo;
	}
}

void UaRdc_DRDlgDestroy(WINDOW win)
{
	RescList_Destroy(IR_FILE, &Rdc_wsList);
	RESC_LIST_INIT(&Rdc_wsList);
	RESC_ASN_NO_MALLOC(&Rdc_wsList);
}

IR_STATUS UaRdc_DRDlgFetchRemote(WINDOW win, UAEP_DIALOG *info)
{
	if (info->data.display.wsId == IR_UNKNOWN_ID_TYPE)
	{
		xvt_note("Cannot fetch without a work space");
		return(IR_FAIL);
	}
	if (info->data.display.assocId == IR_UNKNOWN_ID_TYPE)
	{
		xvt_note("Target is nolonger connected to this work space");
		return(IR_FAIL);
	}

	/* Execute the fetch dialog. Wait for it to complete. */
	create_res_dialog(WD_MODAL, DLG_502, 
	    EM_ALL, dlg_502_eh, (long)info);

	if (info->cancelled == FALSE)
	{
		info->cancelled = TRUE;
		return(IR_SUCCESS);
	}
	else
	{
		return(IR_FAIL);
	}

}

IR_STATUS UaRdc_DRDlgDeleteRemote(WINDOW win, UAEP_DIALOG *info)
{
	/* Execute the remote delete dialog. Wait for it to complete. */
	create_res_dialog(WD_MODAL, DLG_DELETE_REMOTE, 
	    EM_ALL, dlg_111_eh, (long)info);

	if (info->cancelled == FALSE)
	{
		info->cancelled = TRUE;
		return(IR_SUCCESS);
	}
	else
	{
		return(IR_FAIL);
	}
}

IR_STATUS UaRdc_DRDlgDeleteLocal(WINDOW win, UAEP_DIALOG *info)
{
	IRO_DELETE_REQ delReq;

	/* Delete the data in the work space */
	RESC_ASN_NO_MALLOC(&delReq);
	RESC_LIST_INIT(&delReq.wsList);
	RESC_ASN_NO_MALLOC(&delReq.wsList);
	delReq.assocId = info->data.display.assocId;
	delReq.wsId = info->data.display.wsId;
	/* This will destroy the origin process's records. */
	if (IRO_Delete(UaEp_InfoUserId(), IRO_DESTROY_WS, &delReq) 
	    != IR_SUCCESS)
	{
		xvt_note("Could not delete local work space.");
	}
	return(IR_SUCCESS);
}

IR_STATUS UaRdc_DelRemDlgInit(WINDOW win, UAEP_DIALOG *info)
{
	CHAR    tmpBuf[IR_LEN_100];
	WINDOW  ctlWin;
	ctlWin = get_ctl_window(win, DEL_DLG_REMOTE_DISP_RESULT);
	VSToStrNullTerm(tmpBuf, &info->data.display.wsHdrInfo->name, IR_LEN_50);
	set_title(ctlWin, tmpBuf);
	set_front_window(get_ctl_window(win, DEL_DLG_REMOTE_OK));
	return(IR_SUCCESS);
}

IR_STATUS UaRdc_DelRemDlgOk(WINDOW win, UAEP_DIALOG *info)
{
	void UaRdc_ListDeleteWsId(CHAR *src, void *data);
	void *UaRdc_ListCopyWsId(CHAR *src, void *data);
	IRO_DELETE_REQ delReq;
	IRO_WS_HEADER_INFO      *wsInfo;
	IRO_WORK_SPACE_ID       *wsId;

	RESC_ASN_NO_MALLOC(&delReq);
	RESC_LIST_INIT(&delReq.wsList);
	RESC_ASN_NO_MALLOC(&delReq.wsList);

	wsInfo = info->data.display.wsHdrInfo;
	wsId = (IRO_WORK_SPACE_ID *)RescIR_Malloc(IR_FILE,
	    sizeof(IRO_WORK_SPACE_ID));

	*wsId = info->data.display.wsId;

	delReq.deleteFunc = IR_DELETE_SPECIFIC_SET;
	delReq.assocId = wsInfo->assocId;
	RescList_Append(IR_FILE, &delReq.wsList, wsId,
	    UaRdc_ListDeleteWsId, UaRdc_ListCopyWsId);
	
	if (IRO_Delete(UaEp_InfoUserId(), IRO_DELETE_TARGET_RSLTS, 
	    &delReq) == IR_FAIL)
	{
		UaHec_Message("API Error", "Delete remote request failed", TRUE);
		info->cancelled = TRUE;
		RescList_Destroy(IR_FILE, &delReq.wsList);
		return(IR_FAIL);
	}
	else
	{
		info->cancelled = TRUE;
		RescList_Destroy(IR_FILE, &delReq.wsList);
		UaAc_CommStatusUpdate();
		return(IR_SUCCESS);
	}
}
void UaRdc_ListDeleteWsId(CHAR *src, void *data)
{
	RescIR_Free(src, (CHAR *)data);
}

void *UaRdc_ListCopyWsId(CHAR *src, void *data)
{
	IRO_WORK_SPACE_ID *new;
	IRO_WORK_SPACE_ID *from;

	new = (IRO_WORK_SPACE_ID *)RescIR_Malloc(src,
	    sizeof(IRO_WORK_SPACE_ID));

	from = (IRO_WORK_SPACE_ID *)data;

	*new = *from;

	return(new);
}
	
